@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -445,10 +445,15 @@ int scheme_auth(char *procname, radiusd_request_t *req,
                 grad_avp_t *user_check, grad_avp_t **user_reply_ptr);
 int scheme_acct(char *procname, radiusd_request_t *req);
 int scheme_eval_boolean_expr(char *expr);
+int scheme_eval_avl (radiusd_request_t *request,
+		     grad_avp_t *lhs, grad_avp_t *rhs,
+		     grad_avp_t **reply,
+		     grad_avp_t **pfailed);
 void scheme_eval_unspecified_expr(char *expr);
 void scheme_read_eval_loop();
 void scheme_redirect_output();
 void start_guile();
+
 int guile_cfg_handler(int argc, cfg_value_t *argv,
 		      void *block_data, void *handler_data);
 extern struct cfg_stmt guile_stmt[];
@@ -460,7 +465,7 @@ void radiusd_logger(int level,
 		    const char *func_name,
 		    int en,
 		    const char *fmt, va_list ap);
-void sqllog(int status, char *msg, ...);
+void sqllog(int status, char *query);
 int logging_stmt_handler(int argc, cfg_value_t *argv, void *block_data,
 			 void *handler_data);
 int logging_stmt_end(void *block_data, void *handler_data);
@@ -579,6 +584,8 @@ char *make_server_ident();
 void dump_users_db();
 void strip_username(int do_strip, char *name,
                     grad_avp_t *check_item, char *stripped_name);
+int exec_program_wait (radiusd_request_t *request, grad_avp_t *rhs,
+		       grad_avp_t **reply, grad_avp_t **pfailed);
 
 /* version.c */
 void version();
@@ -635,6 +642,8 @@ void radiusd_request_free(radiusd_request_t *radreq);
 
 char *radius_xlate(struct obstack *obp, char *str,
                    grad_request_t *req, grad_avp_t *reply_pairs);
+int radius_eval_avp(radiusd_request_t *req, grad_avp_t *p, grad_avp_t *reply,
+		    int allow_xlat);
 int radius_eval_avl(radiusd_request_t *req, grad_avp_t *p);
 char *util_xlate(struct obstack *sp, char *fmt, grad_request_t *radreq);
 
@@ -706,7 +715,7 @@ void log_release();
 void register_channel(Channel *chan);
 void register_category(int cat, int pri, grad_list_t *chanlist);
 
-void log_set_to_console();
+void log_set_to_console(int cat, int pri);
 void log_set_default(char *name, int cat, int pri);
 
 void log_open(int cat);
@@ -737,7 +746,7 @@ SCM sql_exec_query(int type, char *query);
 # define radiusd_sql_shutdown()
 # define radiusd_sql_clear_cache()
 # define radiusd_sql_acct(req)
-# define radiusd_sql_checkgroup(req, groupname) 0
+# define radiusd_sql_checkgroup(req, groupname) 1
 # define radiusd_sql_check_attr_query(req, check_pairs) 0
 # define radiusd_sql_reply_attr_query(req, reply_pairs)
 # define radiusd_sql_auth_result_query(req, fail)
