@@ -1,5 +1,6 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005,
+   2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -142,7 +143,7 @@ grad_malloc(size_t size)
 
         p = malloc(size + EXTRA);
 
-	if (debug_on(10)) 
+	if (GRAD_DEBUG_LEVEL(10)) 
 		printf("malloc(%d) = %p\n", size, p);
         
         if (p) {
@@ -209,10 +210,10 @@ grad_free(void *ptr)
         mallocstat.size -= mhdr->s.size;
         mallocstat.count--;
         
-        if (debug_on(10))
+        if (GRAD_DEBUG_LEVEL(10))
 		printf("free(%p) %d bytes\n", mhdr, mhdr->s.size);
 #else
-        if (debug_on(10))
+        if (GRAD_DEBUG_LEVEL(10))
 		printf("free(%p)\n", ptr);
 #endif
         free(ptr);
