@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2003 Free Software Foundation
+   Copyright (C) 2003,2004 Free Software Foundation
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,17 +11,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 #include <sys/types.h>
 #include <stdlib.h>
-#include <mem.h>
-#include <list.h>
+#include <radius/radius.h>
+#include <radius/list.h>
 
 struct list_entry {
 	struct list_entry *next;
@@ -36,22 +37,22 @@ struct list {
 
 struct iterator {
 	struct iterator *next;
-	RAD_LIST *list;
+	grad_list_t *list;
 	struct list_entry *cur;
 	int advanced;
 };
 
 struct list *
-list_create()
+grad_list_create()
 {
-	struct list *p = emalloc(sizeof(*p));
+	struct list *p = grad_emalloc(sizeof(*p));
 	p->head = p->tail = NULL;
 	p->itr = NULL;
 	return p;
 }
 
 void
-list_destroy(struct list **plist, list_iterator_t user_free, void *data)
+grad_list_destroy(struct list **plist, list_iterator_t user_free, void *data)
 {
 	struct list_entry *p;
 
@@ -63,83 +64,99 @@ list_destroy(struct list **plist, list_iterator_t user_free, void *data)
 		struct list_entry *next = p->next;
 		if (user_free)
 			user_free(p->data, data);
-		efree(p);
+		grad_free(p);
 		p = next;
 	}
-	efree(*plist);
+	grad_free(*plist);
 	*plist = NULL;
 }
 
 void *
-iterator_current(ITERATOR *ip)
+grad_iterator_current(grad_iterator_t *ip)
 {
 	if (!ip)
 		return NULL;
 	return ip->cur ? ip->cur->data : NULL;
 }
 
-ITERATOR *
-iterator_create(RAD_LIST *list)
+static void
+grad_iterator_attach(grad_iterator_t *itr, grad_list_t *list)
 {
-	ITERATOR *itr;
-
-	if (!list)
-		return NULL;
-	itr = emalloc(sizeof(*itr));
 	itr->list = list;
 	itr->cur = NULL;
 	itr->next = list->itr;
 	itr->advanced = 0;
-	list->itr = itr;
+	list->itr = itr;	
+}
+
+static grad_iterator_t *
+grad_iterator_detach(grad_iterator_t *iter)
+{
+	grad_iterator_t *cur, *prev;
+
+	for (cur = iter->list->itr, prev = NULL;
+	     cur;
+	     prev = cur, cur = cur->next)
+		if (cur == iter)
+			break;
+	
+	if (cur) {
+		if (prev)
+			prev->next = cur->next;
+		else
+			cur->list->itr = cur->next;
+	}
+	return cur;
+}
+		     
+grad_iterator_t *
+grad_iterator_create(grad_list_t *list)
+{
+	grad_iterator_t *itr;
+
+	if (!list)
+		return NULL;
+	itr = grad_emalloc(sizeof(*itr));
+	grad_iterator_attach(itr, list);
 	return itr;
 }
 
 void
-iterator_destroy(ITERATOR **ip)
+grad_iterator_destroy(grad_iterator_t **ip)
 {
-	ITERATOR *itr, *prev;
-
+	grad_iterator_t *itr;
+	
 	if (!ip || !*ip)
 		return;
-	for (itr = (*ip)->list->itr, prev = NULL;
-	     itr;
-	     prev = itr, itr = itr->next)
-		if (*ip == itr)
-			break;
-	if (itr) {
-		if (prev)
-			prev->next = itr->next;
-		else
-			itr->list->itr = itr->next;
-		efree(itr);
-		*ip = NULL;
-	}
-		
+	itr = grad_iterator_detach(*ip);
+	if (itr)
+		grad_free(itr);
+	*ip = NULL;
 }
 		
 void *
-iterator_first(ITERATOR *ip)
+grad_iterator_first(grad_iterator_t *ip)
 {
 	if (!ip)
 		return NULL;
 	ip->cur = ip->list->head;
 	ip->advanced = 0;
-	return iterator_current(ip);
+	return grad_iterator_current(ip);
 }
 
 void *
-iterator_next(ITERATOR *ip)
+grad_iterator_next(grad_iterator_t *ip)
 {
 	if (!ip || !ip->cur)
 		return NULL;
 	if (!ip->advanced)
 		ip->cur = ip->cur->next;
 	ip->advanced = 0;
-	return iterator_current(ip);
+	return grad_iterator_current(ip);
 }	
 
 static void
-_iterator_advance(ITERATOR *ip, struct list_entry *e)
+_iterator_advance(grad_iterator_t *ip, struct list_entry *e)
 {
 	for (; ip; ip = ip->next) {
 		if (ip->cur == e) {
@@ -150,10 +167,10 @@ _iterator_advance(ITERATOR *ip, struct list_entry *e)
 }
 
 void *
-list_item(struct list *list, size_t n)
+grad_list_item(struct list *list, size_t n)
 {
 	struct list_entry *p;
-	if (n >= list->count)
+	if (!list || n >= list->count)
 		return NULL;
 	for (p = list->head; n > 0 && p; p = p->next, n--)
 		;
@@ -161,7 +178,7 @@ list_item(struct list *list, size_t n)
 }
 
 size_t
-list_count(struct list *list)
+grad_list_count(struct list *list)
 {
 	if (!list)
 		return 0;
@@ -169,13 +186,13 @@ list_count(struct list *list)
 }
 
 void
-list_append(struct list *list, void *data)
+grad_list_append(struct list *list, void *data)
 {
 	struct list_entry *ep;
 
 	if (!list)
 		return;
-	ep = emalloc(sizeof(*ep));
+	ep = grad_emalloc(sizeof(*ep));
 	ep->next = NULL;
 	ep->data = data;
 	if (list->tail)
@@ -187,13 +204,13 @@ list_append(struct list *list, void *data)
 }
 
 void
-list_prepend(struct list *list, void *data)
+grad_list_prepend(struct list *list, void *data)
 {
 	struct list_entry *ep;
 
 	if (!list)
 		return;
-	ep = emalloc(sizeof(*ep));
+	ep = grad_emalloc(sizeof(*ep));
 	ep->data = data;
 	ep->next = list->head;
 	list->head = ep;
@@ -209,7 +226,7 @@ cmp_ptr(const void *a, const void *b)
 }
 
 void *
-list_remove(struct list *list, void *data, list_comp_t cmp)
+grad_list_remove(struct list *list, void *data, list_comp_t cmp)
 {
 	struct list_entry *p, *prev;
 
@@ -236,32 +253,32 @@ list_remove(struct list *list, void *data, list_comp_t cmp)
 	if (p == list->tail)
 		list->tail = prev;
 	
-	efree(p);
+	grad_free(p);
 	list->count--;
 	
 	return data;
 }
 
+/* Note: if modifying this function, make sure it does not allocate any
+   memory! */
 void
-list_iterate(struct list *list, list_iterator_t func, void *data)
+grad_list_iterate(struct list *list, list_iterator_t func, void *data)
 {
-	ITERATOR *itr;
+	grad_iterator_t itr;
 	void *p;
 	
 	if (!list)
 		return;
-	itr = iterator_create(list);
-	if (!itr)
-		return;
-	for (p = iterator_first(itr); p; p = iterator_next(itr)) {
+	grad_iterator_attach(&itr, list);
+	for (p = grad_iterator_first(&itr); p; p = grad_iterator_next(&itr)) {
 		if (func(p, data))
 			break;
 	}
-	iterator_destroy(&itr);
+	grad_iterator_detach(&itr);
 }
 
 void *
-list_locate(struct list *list, void *data, list_comp_t cmp)
+grad_list_locate(struct list *list, void *data, list_comp_t cmp)
 {
 	struct list_entry *cur;
 	if (!list)
@@ -275,7 +292,7 @@ list_locate(struct list *list, void *data, list_comp_t cmp)
 }
 	
 int
-list_insert_sorted(struct list *list, void *data, list_comp_t cmp)
+grad_list_insert_sorted(struct list *list, void *data, list_comp_t cmp)
 {
 	struct list_entry *cur, *prev;
 	
@@ -289,11 +306,11 @@ list_insert_sorted(struct list *list, void *data, list_comp_t cmp)
 			break;
 
 	if (!prev) {
-		list_prepend(list, data);
+		grad_list_prepend(list, data);
 	} else if (!cur) {
-		list_append(list, data);
+		grad_list_append(list, data);
 	} else {
-		struct list_entry *ep = emalloc(sizeof(*ep));
+		struct list_entry *ep = grad_emalloc(sizeof(*ep));
 		ep->data = data;
 		ep->next = cur;
 		prev->next = ep;
