@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,9 +13,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation, 
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -36,30 +39,24 @@
 #include <sys/file.h>
 #include <pwd.h>
 #include <grp.h>
-#include <radius.h>
-#include <checkrad.h>
-#include <obstack1.h>
 
-RADIUS_REQ *
-radreq_alloc()
+#include <common.h>
+
+grad_request_t *
+grad_request_alloc()
 {
-        return emalloc(sizeof(RADIUS_REQ));
+        return grad_emalloc(sizeof(grad_request_t));
 }
 
-/* Free a RADIUS_REQ struct.
+/* Free a grad_request_t struct.
  */
 void 
-radreq_free(RADIUS_REQ *radreq)
+grad_request_free(grad_request_t *radreq)
 {
-	efree(radreq->remote_user);
-        avl_free(radreq->reply_pairs);
-        efree(radreq->reply_msg);
-        avl_free(radreq->server_reply);
-        avl_free(radreq->request);
-        efree(radreq);
+        grad_avl_free(radreq->avlist);
+        grad_free(radreq);
 }
 
-
 /* Turn printable string (dictionary type DATE) into correct tm struct entries
  */
 static char *months[] = {
@@ -67,7 +64,7 @@ static char *months[] = {
         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 
 int
-user_gettime(char *valstr, struct tm *tm)
+grad_parse_time_string(char *valstr, struct tm *tm)
 {
         int     i;
 
@@ -107,22 +104,22 @@ user_gettime(char *valstr, struct tm *tm)
  * `whence' determines from where the offset is counted (seek-like)
  */
 void
-rad_lock(int fd, size_t size, off_t offset, int whence)
+grad_lock_file(int fd, size_t size, off_t offset, int whence)
 {
         struct flock fl;
 
-        fl.l_type = F_RDLCK;
+        fl.l_type = F_WRLCK;
         fl.l_whence = whence;
         fl.l_start = offset;
         fl.l_len = size;
-        fcntl(fd, F_SETLKW, &fl);
+        fcntl(fd, F_SETLKW, &fl); /* FIXME: Handle EINTR */
 }
 
 /* Unlock `size' bytes on the file descriptor `fd' starting from `offset'.
  * `whence' determines from where the offset is counted (seek-like)
  */
 void
-rad_unlock(int fd, size_t size, off_t offset, int whence)
+grad_unlock_file(int fd, size_t size, off_t offset, int whence)
 {
         struct flock fl;
 
@@ -133,11 +130,11 @@ rad_unlock(int fd, size_t size, off_t offset, int whence)
         fcntl(fd, F_SETLKW, &fl);
 }
 
-/* Find a struct keyword matching the given string. Return keyword token
+/* Find a grad_keyword_t matching the given string. Return keyword token
  * number if found. Otherwise return default value `def'.
  */
 int
-xlat_keyword(struct keyword *kw, char *str, int def)
+grad_xlat_keyword(grad_keyword_t *kw, const char *str, int def)
 {
         for ( ; kw->name; kw++) 
                 if (strcmp(str, kw->name) == 0)
@@ -148,10 +145,10 @@ xlat_keyword(struct keyword *kw, char *str, int def)
 /* compose a full pathname from given path and filename
  */
 char *
-mkfilename(char *dir, char *name)
+grad_mkfilename(char *dir, char *name)
 {
         int len = strlen(dir) + strlen(name);
-        char *p = emalloc(len+2);
+        char *p = grad_emalloc(len+2);
         sprintf(p, "%s/%s", dir, name);
         return p;
 }
@@ -159,12 +156,12 @@ mkfilename(char *dir, char *name)
 /* compose a full pathname from given path, subdirectory and filename
  */
 char *
-mkfilename3(char *dir, char *subdir, char *name)
+grad_mkfilename3(char *dir, char *subdir, char *name)
 {
         int len = strlen(dir) + strlen(subdir) + strlen(name);
-        char *p = emalloc(len+3); /* two intermediate slashes and
-                                   * terminating zero
-                                   */
+        char *p = grad_emalloc(len+3); /* two intermediate slashes and
+                                        * terminating zero
+                                        */
         sprintf(p, "%s/%s/%s", dir, subdir, name);
         return p;
 }
@@ -172,7 +169,7 @@ mkfilename3(char *dir, char *subdir, char *name)
 /* Convert second character of a backslash sequence to its ASCII
    value: */
 int
-backslash(int c)
+grad_decode_backslash(int c)
 {
         static char transtab[] = "a\ab\bf\fn\nr\rt\t";
         char *p;
@@ -185,117 +182,114 @@ backslash(int c)
 }
 
 #define to_num(c) \
-  (isdigit(c) ? c - '0' : (isxdigit(c) ? toupper(c) - 'A' + 10 : -1 ))
+  (isdigit(c) ? c - '0' : (isxdigit(c) ? toupper(c) - 'A' + 10 : 255 ))
 
 void
-obstack_grow_backslash_num(struct obstack *stk, char *text, int len, int base)
+grad_obstack_grow_backslash_num(struct obstack *stk, char *text, char **pend,
+				int len, int base)
 {
 	int i;
-	int c = 0;
-
-	for (i = len-1; i >= 0 && text[i] != '\\'; i--)
-		;
-	if (i)
-		obstack_grow(stk, text, i);
-	if (base == 16)
-		i++;
-	for (i++; i < len; i++) 
-		c = c*base + to_num(text[i]);
-	obstack_1grow(stk, c);
+	int val = 0;
+	char *start = text;
+	
+	if (text[0] == '\\') {
+		text++;
+		if (base == 16)
+			text++;
+	}
+	
+	for (i = 0; i < len; i++) {
+		int n = (unsigned char)text[i];
+		if (n > 127 || (n = to_num(n)) >= base)
+			break;
+		val = val*base + n;
+	}
+	
+	if (i == 0) {
+		obstack_grow(stk, start, 1);
+		if (pend)
+			*pend = start + 1;
+	} else {
+		obstack_1grow(stk, val);
+		if (pend)
+			*pend = text + i;
+	}
 }
 
 
 void
-obstack_grow_backslash(struct obstack *stk, char *text, char **endp)
+grad_obstack_grow_backslash(struct obstack *stk, char *text, char **endp)
 {
-	int c, i;
-	
-	switch (text[1]) {
-	case '\\':
+	if (text[1] == '\\' || (unsigned char)text[1] > 127) {
 		obstack_1grow(stk, text[1]);
 		text += 2;
-		break;
-		
-	default:
-		c = backslash(text[1]);
+	} else if (isdigit(text[1])) 
+		grad_obstack_grow_backslash_num(stk, text, &text, 3, 8);
+	else if (text[1] == 'x' || text[1] == 'X')
+		grad_obstack_grow_backslash_num(stk, text, &text, 2, 16);
+	else {
+		int c = grad_decode_backslash(text[1]);
 		obstack_1grow(stk, c);
 		text += 2;
-		break;
-		
-	case '0':
-		for (i = 0; i < 3 && isdigit(text[i+1]) && text[i+1] < '8'; i++)
-			;
-		if (i != 3)
-			break;
-		obstack_grow_backslash_num(stk, text, 4, 8);
-		text += 4;
-		break;
-		
-	case 'x':
-	case 'X':
-		for (i = 0; i < 2 && isxdigit(text[i+2]); i++)
-			;
-		if (i != 2)
-			break;
-		obstack_grow_backslash_num(stk, text, 3, 16);
-		text += 3;
-		break;
 	}
+		
 	*endp = text;
 }
 
 void
-string_copy(char *d, char *s, int len)
+grad_string_copy(char *d, char *s, int len)
 {
         int slen = strlen(s);
         
         if (slen > len) 
-                radlog(L_ERR, _("string too long: %s"), s);
+                grad_log(L_ERR, _("string too long: %s"), s);
         strncpy(d, s, len);
         d[len] = 0;
 }
 
 char *
-op_to_str(int op)
+grad_op_to_str(enum grad_operator op)
 {
         switch (op) {
-        case OPERATOR_EQUAL:         return "=";
-        case OPERATOR_NOT_EQUAL:     return "!=";
-        case OPERATOR_LESS_THAN:     return "<";
-        case OPERATOR_GREATER_THAN:  return ">";
-        case OPERATOR_LESS_EQUAL:    return "<=";
-        case OPERATOR_GREATER_EQUAL: return ">=";
+        case grad_operator_equal:         return "=";
+        case grad_operator_not_equal:     return "!=";
+        case grad_operator_less_than:     return "<";
+        case grad_operator_greater_than:  return ">";
+        case grad_operator_less_equal:    return "<=";
+        case grad_operator_greater_equal: return ">=";
+	default:
+		break;
         }
         return "?";
 }
 
-int
-str_to_op(char *str)
+enum grad_operator
+grad_str_to_op(char *str)
 {
-        int op = NUM_OPERATORS;
+        int op = grad_operator_invalid;
         switch (*str++) {
         case '=':
-                op = OPERATOR_EQUAL;
+                op = grad_operator_equal;
                 break;
         case '!':
                 if (*str++ == '=')
-                        op = OPERATOR_NOT_EQUAL;
+                        op = grad_operator_not_equal;
                 break;
         case '<':
                 if (*str == 0)
-                        op = OPERATOR_LESS_THAN;
+                        op = grad_operator_less_than;
                 else if (*str++ == '=')
-                        op = OPERATOR_LESS_EQUAL;
+                        op = grad_operator_less_equal;
                 break;
         case '>':
                 if (*str == 0)
-                        op = OPERATOR_GREATER_THAN;
+                        op = grad_operator_greater_than;
                 else if (*str++ == '=')
-                        op = OPERATOR_GREATER_EQUAL;
+                        op = grad_operator_greater_equal;
                 break;
         }
         if (*str)
-                op = NUM_OPERATORS;
+                op = grad_operator_invalid;
         return op;
 }
 
@@ -332,7 +326,7 @@ flush_seg(char **bufp, char *seg, char *ptr, int runlen)
    (not counting null terminator) is returned. */
 
 int
-format_string_visual(char *buf, int runlen, char *str, int len)
+grad_format_string_visual(char *buf, int runlen, char *str, int len)
 {
         char *seg, *ptr;
         int outbytes = 0;
@@ -370,10 +364,10 @@ format_string_visual(char *buf, int runlen, char *str, int len)
 }
 
 int
-format_vendor_pair(char *buf, VALUE_PAIR *pair)
+grad_format_vendor_pair(char *buf, grad_avp_t *pair)
 {
         int n;
-        UINT4 vendor;
+        grad_uint32_t vendor;
         u_char *ptr = (u_char*)pair->avp_strvalue;
         char buf1[64];
         char *bufp = buf;
@@ -389,49 +383,51 @@ format_vendor_pair(char *buf, VALUE_PAIR *pair)
                 bufp += n;
         }
         
-        return n + format_string_visual(bufp, 4, ptr, pair->avp_strlength - 4);
+        return n + grad_format_string_visual(bufp, 4, ptr,
+					     pair->avp_strlength - 4);
 }
                 
 char *
-format_pair(VALUE_PAIR *pair, char **savep)
+grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
 {
         char *buf1 = NULL;
         char *buf2ptr = NULL;
-        char buf2[4*AUTH_STRING_LEN+1]; /* Enough to hold longest possible
+        char buf2[4*GRAD_STRING_LENGTH+1]; /* Enough to hold longest possible
                                            string value all converted to
                                            octal */
-        DICT_VALUE *dval;
+        grad_dict_value_t *dval;
         struct tm tm;
-        
+        char *type = "";
+	
         *savep = NULL;
 
-        switch (pair->eval ? TYPE_STRING : pair->type) {
-        case TYPE_STRING:
+        switch (pair->eval_type == grad_eval_const ? pair->type : GRAD_TYPE_STRING) {
+        case GRAD_TYPE_STRING:
                 if (pair->attribute != DA_VENDOR_SPECIFIC) {
                         int len = strlen (pair->avp_strvalue);
                         if (len != pair->avp_strlength-1)
                                 len = pair->avp_strlength;
-                        format_string_visual(buf2, 4,
-                                             pair->avp_strvalue, len);
+                        grad_format_string_visual(buf2, 4,
+						  pair->avp_strvalue, len);
                 } else if (pair->avp_strlength < 6) 
                         snprintf(buf2, sizeof(buf2),
                                  "[invalid length: %d]", pair->avp_strlength);
                 else {
-                        int len = format_vendor_pair(NULL, pair);
+                        int len = grad_format_vendor_pair(NULL, pair);
                         buf2ptr = malloc(len+1);
                         if (!buf2ptr) {
-                                radlog(L_ERR,
-                                       "%s:%d: can't alloc %d bytes",
-                                       __FILE__, __LINE__, len+1);
+                                grad_log(L_ERR,
+                                         "%s:%d: can't alloc %d bytes",
+                                         __FILE__, __LINE__, len+1);
                                 buf2[0] = 0;
                         } else
-                                format_vendor_pair(buf2ptr, pair);
+                                grad_format_vendor_pair(buf2ptr, pair);
                 }
                 break;
                                         
-        case TYPE_INTEGER:
-                if (pair->name)
-                        dval = value_lookup(pair->avp_lvalue, pair->name);
+        case GRAD_TYPE_INTEGER:
+                if (pair->name && (pair->prop & GRAD_AP_TRANSLATE))
+                        dval = grad_value_lookup(pair->avp_lvalue, pair->name);
                 else
                         dval = NULL;
                 
@@ -441,11 +437,11 @@ format_pair(VALUE_PAIR *pair, char **savep)
                         snprintf(buf2, sizeof(buf2), "%s", dval->name);
                 break;
                 
-        case TYPE_IPADDR:
-                ip_iptostr(pair->avp_lvalue, buf2);
+        case GRAD_TYPE_IPADDR:
+                grad_ip_iptostr(pair->avp_lvalue, buf2);
                 break;
                 
-        case TYPE_DATE:
+        case GRAD_TYPE_DATE:
                 strftime(buf2, sizeof(buf2), "\"%b %e %Y\"",
                          localtime_r((time_t *)&pair->avp_lvalue, &tm));
                 break;
@@ -453,15 +449,37 @@ format_pair(VALUE_PAIR *pair, char **savep)
                 strncpy(buf2, "[UNKNOWN DATATYPE]", sizeof(buf2));
         }
 
+	if (typeflag) {
+		switch (pair->type) {
+		case GRAD_TYPE_STRING:
+			type = "(STRING) ";
+			break;
+			
+		case GRAD_TYPE_INTEGER:
+			type = "(INTEGER) ";
+			break;
+			
+		case GRAD_TYPE_IPADDR:
+			type = "(IPADDR) ";
+			break;
+			
+		case GRAD_TYPE_DATE:
+			type = "(DATE) ";
+			break;
+		}
+	}
+	
         if (pair->name)
-                asprintf(&buf1, "%s %s %s",
+                asprintf(&buf1, "%s %s %s%s",
                          pair->name,
-                         op_to_str(pair->operator),
+                         grad_op_to_str(pair->operator),
+			 type,
                          buf2ptr ? buf2ptr : buf2);
         else
-                asprintf(&buf1, "%d %s %s",
+                asprintf(&buf1, "%d %s %s%s",
                          pair->attribute,
-                         op_to_str(pair->operator),
+                         grad_op_to_str(pair->operator),
+			 type,
                          buf2ptr ? buf2ptr : buf2);
 
         if (buf2ptr)
