@@ -1,5 +1,6 @@
 /* This file is part of GNU Radius
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005,
+   2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -96,8 +97,8 @@ create_instance(struct check_instance *cptr, grad_nas_t *nas, struct radutmp *up
         cptr->name = up->orig_login[0] ? up->orig_login : up->login;
         cptr->port = up->nas_port;
         cptr->sid  = up->session_id;
-        cptr->framed_ip = up->framed_address;
-	cptr->nas_ip = up->nas_address;
+        cptr->framed_ip = ntohl(up->framed_address);
+	cptr->nas_ip = ntohl(up->nas_address);
 	cptr->result = -1;
         cptr->timeout = 0;
         cptr->nasname = nas->shortname ? nas->shortname : nas->longname;
@@ -120,9 +121,9 @@ free_instance(struct check_instance *cptr)
 int
 compare(struct check_instance *checkp, char *str)
 {
-	grad_uint32_t result;
+	grad_value_t val;
 
-	if (rewrite_invoke(Integer, &result,
+	if (rewrite_invoke(Integer, &val,
 			   checkp->func, NULL,
 			   "ssis",
 			   str,
@@ -130,7 +131,7 @@ compare(struct check_instance *checkp, char *str)
 			   checkp->port,
 			   checkp->sid))
 		return -1;
-	return result;
+	return val.datum.ival;
 }
 
 
@@ -255,19 +256,19 @@ converse(int type, struct snmp_session *sp, struct snmp_pdu *pdu, void *closure)
                 switch (vlist->type) {
                 case SMI_STRING:
                         rc = compare(checkp, vlist->var_str);
-                        debug(2, ("(STRING) %s: %d", vlist->var_str, rc));
+                        GRAD_DEBUG(2, ("(STRING) %s: %d", vlist->var_str, rc));
                         break;
                 case SMI_INTEGER:
                 case SMI_COUNTER32:
                 case SMI_COUNTER64:
                         snprintf(buf, sizeof(buf), "%d", vlist->var_int);
                         rc = compare(checkp, buf);
-                        debug(2, ("(INT) %d: %d", vlist->var_int, rc));
+                        GRAD_DEBUG(2, ("(INT) %d: %d", vlist->var_int, rc));
                         break;
                 case SMI_IPADDRESS:
                         grad_ip_iptostr(*(grad_uint32_t*)vlist->var_int, buf);
                         rc = compare(checkp, buf);
-                        debug(2, ("(IPADDR) %#x: %d",
+                        GRAD_DEBUG(2, ("(IPADDR) %#x: %d",
                                   *(grad_uint32_t*)vlist->var_str, rc));
                         break;
                 }
@@ -355,7 +356,7 @@ snmp_check(struct check_instance *checkp, grad_nas_t *nas)
         snmp_pdu_add_var(pdu, var);
         snmp_free(oid);
 
-        debug(1, ("snmpget: %s:%d %s %s",
+        GRAD_DEBUG(1, ("snmpget: %s:%d %s %s",
                   peername,
                   remote_port,
                   community,
@@ -368,7 +369,7 @@ snmp_check(struct check_instance *checkp, grad_nas_t *nas)
         rc = checkp->result;
         snmp_session_close(sp);
         
-        debug(1, ("result: %d", rc));
+        GRAD_DEBUG(1, ("result: %d", rc));
         return rc;
 }
 
@@ -439,7 +440,7 @@ finger_check(struct check_instance *checkp, grad_nas_t *nas)
                 return -1;
         }
 
-        debug(1, ("finger %s@%s:%d",
+        GRAD_DEBUG(1, ("finger %s@%s:%d",
                   namebuf, hp->h_name, ntohs(port)));
 
         /* have network connection; identify the host connected with */
@@ -520,7 +521,7 @@ finger_check(struct check_instance *checkp, grad_nas_t *nas)
                                 
                                 obstack_1grow(&checkp->stack, 0);
                                 ptr = obstack_finish(&checkp->stack);
-                                debug(2,("got : %s", ptr));
+                                GRAD_DEBUG(2,("got : %s", ptr));
                                 found = compare(checkp, ptr);
                                 obstack_free(&checkp->stack, ptr);
                                 if (found) 
@@ -541,7 +542,7 @@ finger_check(struct check_instance *checkp, grad_nas_t *nas)
 
                         obstack_1grow(&checkp->stack, '\n');
                         obstack_1grow(&checkp->stack, 0);
-                        debug(2,("got : %s", ptr));
+                        GRAD_DEBUG(2,("got : %s", ptr));
                         ptr = obstack_finish(&checkp->stack);
                         found = compare(checkp, ptr);
                 }
@@ -555,7 +556,7 @@ finger_check(struct check_instance *checkp, grad_nas_t *nas)
         /* restore alarm settings */
         alarm(0);
 
-        debug(1, ("result: %d", found));
+        GRAD_DEBUG(1, ("result: %d", found));
         checkp->result = found;
         return found;
 }
