@@ -1,5 +1,6 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005,
+   2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -321,20 +322,20 @@ user_find_sym(char *name, radiusd_request_t *req,
         User_symbol *sym;
         USER_LOOKUP lu;
         
-        debug(1,("looking for %s", name));
+        GRAD_DEBUG(1,("looking for %s", name));
         for (sym = user_lookup(name, &lu); sym; sym = user_next(&lu)) {
                 if (match_user(sym, req, check_pairs, reply_pairs)) {
 			if (lu.state != LU_begin)
 				found = 1;
                         if (!fallthrough(sym->reply))
                                 break;
-                        debug(1, ("fall through"));
+                        GRAD_DEBUG(1, ("fall through"));
                         lu.sym = NULL; /* force jump to next state */
                 }
         }
 	if (found)
 		radiusd_sql_reply_attr_query(req, reply_pairs);
-        debug(1, ("returning %d", found));
+        GRAD_DEBUG(1, ("returning %d", found));
         return found;
 }
 
@@ -365,10 +366,10 @@ match_user(User_symbol *sym, radiusd_request_t *req,
                 for (p = grad_avl_find(sym->check, DA_MATCH_PROFILE);
                      p; 
                      p = grad_avl_find(p->next, DA_MATCH_PROFILE)) {
-                        debug(1, ("submatch: %s", p->avp_strvalue));
+                        GRAD_DEBUG(1, ("submatch: %s", p->avp_strvalue));
 
                         found = match_user(grad_sym_lookup(user_tab,
-						      p->avp_strvalue),
+							   p->avp_strvalue),
                                            req, check_pairs, reply_pairs);
                 }                       
 
@@ -387,13 +388,13 @@ match_user(User_symbol *sym, radiusd_request_t *req,
                 for (p = grad_avl_find(sym->reply, DA_MATCH_PROFILE);
                      p;
                      p = grad_avl_find(p->next, DA_MATCH_PROFILE)) {
-                        debug(1, ("next: %s", p->avp_strvalue));
+                        GRAD_DEBUG(1, ("next: %s", p->avp_strvalue));
                         match_user(grad_sym_lookup(user_tab, p->avp_strvalue),
                                    req, check_pairs, reply_pairs);
                 }
                 if (!fallthrough(sym->reply))
                         break;
-                debug(1, ("fall through near %s:%lu",
+                GRAD_DEBUG(1, ("fall through near %s:%lu",
 			  sym->loc.file, (unsigned long) sym->loc.line));
         } while (sym = grad_sym_next((grad_symbol_t*)sym));
 
@@ -607,8 +608,7 @@ avl_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule,
 {
 	for (; p; p = p->next) {
 		if (p->attribute == DA_REWRITE_FUNCTION
-		    && (rewrite_eval(p->avp_strvalue, req->request,
-				     NULL, NULL))) {
+		    && rewrite_eval(p->avp_strvalue, req->request, NULL)) {
 			grad_log_loc(L_ERR, &rule->loc, "%s(): %s",
 				     p->avp_strvalue,
 				     _("not defined"));
@@ -623,6 +623,47 @@ rule_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule)
 	avl_eval_rewrite (req, rule, rule->lhs);
 }
 
+int
+exec_program_wait (radiusd_request_t *request, grad_avp_t *rhs,
+		   grad_avp_t **reply, grad_avp_t **pfailed)
+{
+	int rc = 0;
+	grad_avp_t *p;
+	
+	for (p = rhs; p; p = p->next) {
+		if (p->attribute == DA_EXEC_PROGRAM_WAIT) {
+			radius_eval_avp(request, p, reply ? *reply : NULL, 1);
+			switch (p->avp_strvalue[0]) {
+			case '/':
+				rc = radius_exec_program(p->avp_strvalue,
+							 request,
+							 reply,
+							 1);
+				break;
+				
+			case '|':
+				rc = filter_auth(p->avp_strvalue+1,
+						 request,
+						 reply);
+				break;
+
+			default:
+				grad_log_req(L_ERR, request->request,
+					     _("cannot execute `%s'"),
+					     p->avp_strvalue);
+				rc = 1;
+			}
+
+			if (rc) {
+				if (pfailed)
+					*pfailed = p;
+				break;
+			}
+		}
+	}
+	return rc;
+}
+
 /* ***************************************************************************
  * raddb/hints
  */
@@ -630,9 +671,10 @@ rule_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule)
 /* Provide a support for backward-compatible attributes Replace-User-Name
    and Rewrite-Function */
 static void
-hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair, grad_matching_rule_t *rule)
+hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair,
+		  grad_matching_rule_t *rule)
 {
-        grad_avp_t      *tmp;
+        grad_avp_t *tmp;
 	
 	/* Let's see if we need to further modify the username */
 	if ((tmp = grad_avl_find(rule->rhs, DA_REPLACE_USER_NAME))
@@ -657,12 +699,13 @@ hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair, grad_matching_r
 int
 hints_setup(radiusd_request_t *req)
 {
-        char            newname[GRAD_STRING_LENGTH];
-        grad_avp_t      *name_pair;
-        grad_avp_t      *orig_name_pair;
-        grad_avp_t      *tmp;
-        grad_matching_rule_t   *rule;
-        int             matched = 0;
+        char newname[GRAD_STRING_LENGTH];
+        grad_avp_t *name_pair;
+        grad_avp_t *orig_name_pair;
+	grad_avp_t *tmp_name_pair;
+        grad_avp_t *tmp;
+        grad_matching_rule_t *rule;
+        int matched = 0;
 	grad_iterator_t *itr;
 	
         /* Add Proxy-Replied pair if necessary */
@@ -679,23 +722,23 @@ hints_setup(radiusd_request_t *req)
         case RT_STATUS_SERVER:
                 return 0;
         }
-        
+       
         if (hints == NULL)
                 return 0;
 
-        /* 
-         *      Check for valid input, zero length names not permitted 
-         */
+        /* Get User-Name pair. If it is absent, provide a replacement
+	   value. */
         if ((name_pair = grad_avl_find(req->request->avlist,
 				       DA_USER_NAME)) == NULL) {
-                name_pair = grad_avp_create_string(DA_USER_NAME, "");
+		tmp_name_pair = grad_avp_create_string(DA_USER_NAME, "");
+                name_pair = tmp_name_pair;
                 orig_name_pair = NULL;
         } else {
                 orig_name_pair = grad_avp_create_string(DA_ORIG_USER_NAME,
 						      name_pair->avp_strvalue);
         }
 
-        debug(1, ("called for `%s'", name_pair->avp_strvalue));
+        GRAD_DEBUG(1, ("called for `%s'", name_pair->avp_strvalue));
         
         /* if Framed-Protocol is present but Service-Type is missing, add
            Service-Type = Framed-User. */
@@ -718,7 +761,7 @@ hints_setup(radiusd_request_t *req)
                 matched++;
                 radius_req_register_locus(req, &rule->loc);
 		
-                debug(1, ("matched %s at %s:%lu", rule->name, rule->loc.file,
+                GRAD_DEBUG(1, ("matched %s at %s:%lu", rule->name, rule->loc.file,
 			  (unsigned long) rule->loc.line));
         
                 add = grad_avl_dup(rule->rhs);
@@ -736,13 +779,14 @@ hints_setup(radiusd_request_t *req)
 
 		/* Handle Rewrite-Function and Replace-User-Name */
 		hints_eval_compat(req, name_pair, rule);
+		/* Process eventual Exec-Program-Wait and Scheme-Procedure
+		   attributes */
+		exec_program_wait (req, rule->rhs, &add, NULL);
+#ifdef USE_SERVER_GUILE
+		scheme_eval_avl (req, rule->lhs, rule->rhs, &add, NULL);
+#endif
 		/* Evaluate hints */
 		radius_eval_avl(req, add); /*FIXME: return value?*/
-		
-                debug(1, ("new name is `%s'", name_pair->avp_strvalue));
-
-                /* fix-up the string length */
-                name_pair->avp_strlength = strlen(name_pair->avp_strvalue);
 
                 /* Add all attributes to the request list, except
 		   DA_STRIP_USER_NAME and DA_REPLACE_USER_NAME */
@@ -752,6 +796,16 @@ hints_setup(radiusd_request_t *req)
                 grad_avl_merge(&req->request->avlist, &add);
                 grad_avl_free(add);
                 
+		/* Re-read the name pair, since it might have been changed
+		   by exec_program_wait */
+		tmp = grad_avl_find(req->request->avlist, DA_USER_NAME);
+		if (tmp)
+			name_pair = tmp;
+                GRAD_DEBUG(1, ("new name is `%s'", name_pair->avp_strvalue));
+
+                /* fix-up the string length. FIXME: Do we still need it? */
+                name_pair->avp_strlength = strlen(name_pair->avp_strvalue);
+
                 /* Ok, let's see if we need to further check the
                    hint's rules */
                 if (((tmp = grad_avl_find(rule->rhs, DA_FALL_THROUGH))
@@ -769,17 +823,18 @@ hints_setup(radiusd_request_t *req)
                 if (orig_name_pair)
                         grad_avl_add_pair(&req->request->avlist,
 					  orig_name_pair);
-		/* A rewrite function might have installed the new
+		/* A rewrite function might have installed a new
 		   User-Name. Take care not to discard it. */
                 else if (!grad_avl_find(req->request->avlist, DA_USER_NAME))
-                        grad_avl_add_pair(&req->request->avlist, name_pair);
+                        grad_avl_add_pair(&req->request->avlist,
+					  tmp_name_pair);
 		else
-			grad_avp_free(name_pair);
+			grad_avp_free(tmp_name_pair);
         } else {
                 if (orig_name_pair)
                         grad_avp_free(orig_name_pair);
                 else
-                        grad_avp_free(name_pair);
+                        grad_avp_free(tmp_name_pair);
         }
 
         return 0;
@@ -805,7 +860,7 @@ huntgroup_match(radiusd_request_t *req, char *huntgroup)
                 if (strcmp(rule->name, huntgroup) != 0)
                         continue;
                 if (paircmp(req, rule->lhs, NULL) == 0) {
-                        debug(1, ("matched %s at %s:%lu",
+                        GRAD_DEBUG(1, ("matched %s at %s:%lu",
 				  rule->name, rule->loc.file,
 				  (unsigned long) rule->loc.line));
                         break;
@@ -840,7 +895,7 @@ huntgroup_access(radiusd_request_t *radreq, grad_locus_t *loc)
                  */
                 if (paircmp(radreq, rule->lhs, NULL) != 0)
                         continue;
-                debug(1, ("matched huntgroup at %s:%lu", rule->loc.file,
+                GRAD_DEBUG(1, ("matched huntgroup at %s:%lu", rule->loc.file,
 			  (unsigned long) rule->loc.line));
                 r = paircmp(radreq, rule->rhs, NULL) == 0;
                 break;
@@ -853,7 +908,7 @@ huntgroup_access(radiusd_request_t *radreq, grad_locus_t *loc)
 		radius_req_register_locus(radreq, &rule->loc);
 		rule_eval_rewrite (radreq, rule);
 	}
-        debug(1, ("returning %d", r));
+        GRAD_DEBUG(1, ("returning %d", r));
         return r;
 }
 
@@ -935,7 +990,7 @@ client_lookup_ip(grad_uint32_t ipaddr)
 			char ipbuf[GRAD_IPV4_STRING_LENGTH];
 			char maskbuf[GRAD_IPV4_STRING_LENGTH];
 
-			debug(10,
+			GRAD_DEBUG(10,
 			      ("Found secret for %s/%s (%s): %s",
 			       grad_ip_iptostr(cl->netdef.ipaddr, ipbuf),
 			       grad_ip_iptostr(cl->netdef.netmask, maskbuf),
@@ -1095,7 +1150,7 @@ read_deny_file()
         int denycnt;
         char *name;
         
-        name = grad_mkfilename(radius_dir, RADIUS_DENY);
+        name = grad_mkfilename(grad_config_dir, RADIUS_DENY);
         if (deny_tab)
                 grad_symtab_clear(deny_tab);
         else
@@ -1222,7 +1277,7 @@ presufcmp(grad_avp_t *check, char *name, char *rest)
         int len, namelen;
         int ret = -1;
 
-        debug(1, ("comparing %s and %s, check->attr is %d",
+        GRAD_DEBUG(1, ("comparing %s and %s, check->attr is %d",
                  name, check->avp_strvalue, check->attribute));
 
         len = strlen(check->avp_strvalue);
@@ -1243,7 +1298,7 @@ presufcmp(grad_avp_t *check, char *name, char *rest)
                         }
                         break;
         }
-        debug(1, ("returning %d", ret));
+        GRAD_DEBUG(1, ("returning %d", ret));
         return ret;
 }
 
@@ -1312,7 +1367,7 @@ paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
                         continue;
                 }
 
-                if (debug_on(20)) {
+                if (GRAD_DEBUG_LEVEL(20)) {
                         grad_log(L_DEBUG, 
                                  "check_item: %s", 
                                  grad_format_pair(check_item, 1, &save));
@@ -1324,7 +1379,7 @@ paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
                  */
                 for (auth_item = request->request->avlist; auth_item; 
                                 auth_item = auth_item->next) {
-                        debug(30, ("trying %d", auth_item->attribute));
+                        GRAD_DEBUG(30, ("trying %d", auth_item->attribute));
 
                         switch (check_item->attribute) {
                         case DA_PREFIX:
@@ -1359,7 +1414,7 @@ paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
                         continue;
                 }
 
-                if (debug_on(20)) {
+                if (GRAD_DEBUG_LEVEL(20)) {
                         grad_log(L_DEBUG,
                                  "auth_item: %s",
                                  grad_format_pair(auth_item, 1, &save));
@@ -1422,7 +1477,7 @@ paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
                         break;
                 }
 
-                debug(20, ("compare: %d", compare));
+                GRAD_DEBUG(20, ("compare: %d", compare));
 
                 result = comp_op(check_item->operator, compare);
 
@@ -1430,7 +1485,7 @@ paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
                         check_item = check_item->next;
         }
 
-        debug(20, ("returning %d", result));
+        GRAD_DEBUG(20, ("returning %d", result));
         return result;
 }
 
@@ -1642,7 +1697,7 @@ reload_data(enum reload_what what, int *do_radck)
 
         case reload_users:
                 grad_symtab_clear(user_tab);
-                path = grad_mkfilename(radius_dir, RADIUS_USERS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_USERS);
         
 #if USE_DBM
                 if (use_dbm && radius_mode != MODE_BUILDDBM) {
@@ -1685,14 +1740,14 @@ reload_data(enum reload_what what, int *do_radck)
                 
         case reload_huntgroups:
                 grad_list_destroy(&huntgroups, matching_rule_free, NULL);
-                path = grad_mkfilename(radius_dir, RADIUS_HUNTGROUPS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_HUNTGROUPS);
                 huntgroups = file_read(GRAD_CF_HUNTGROUPS, path);
                 grad_free(path);
                 break;
                 
         case reload_hints:
                 grad_list_destroy(&hints, matching_rule_free, NULL);
-                path = grad_mkfilename(radius_dir, RADIUS_HINTS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_HINTS);
                 hints = file_read(GRAD_CF_HINTS, path);
                 grad_free(path);
                 if (!use_dbm) 
@@ -1700,7 +1755,7 @@ reload_data(enum reload_what what, int *do_radck)
                 break;
                 
         case reload_clients:
-                path = grad_mkfilename(radius_dir, RADIUS_CLIENTS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_CLIENTS);
                 if (read_clients_file(path) < 0)
                         rc = 1;
                 grad_free(path);
@@ -1708,19 +1763,19 @@ reload_data(enum reload_what what, int *do_radck)
 
         case reload_naslist:
                 /*FIXME*/
-		path = grad_mkfilename(radius_dir, RADIUS_NASTYPES);
+		path = grad_mkfilename(grad_config_dir, RADIUS_NASTYPES);
                 read_nastypes_file(path);
                 grad_free(path);
                 /*EMXIF*/
 
-                path = grad_mkfilename(radius_dir, RADIUS_NASLIST);
+                path = grad_mkfilename(grad_config_dir, RADIUS_NASLIST);
                 if (read_naslist_file(path) < 0)
                         rc = 1;
                 grad_free(path);
                 break;
 
         case reload_realms:
-                path = grad_mkfilename(radius_dir, RADIUS_REALMS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_REALMS);
                 if (grad_read_realms(path, auth_port, acct_port,
 				     realm_set_secret) < 0)
                         rc = 1;
@@ -1816,7 +1871,7 @@ void
 dump_users_db()
 {
         FILE *fp;
-        char *name = grad_mkfilename(radlog_dir, RADIUS_DUMPDB_NAME);
+        char *name = grad_mkfilename(grad_log_dir, RADIUS_DUMPDB_NAME);
         
         fp = fopen(name, "w");
         if (!fp) {
