@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -28,16 +28,103 @@
 #include <unistd.h>
 #include <errno.h>
 #include <syslog.h>
+
 #include <radiusd.h>
 #include <rewrite.h>
 
 static int logging_category = L_CAT(L_MAIN);
-static RAD_LIST /* of Channel*/ *chanlist;     /* List of defined channels */
+static grad_list_t /* of Channel*/ *chanlist;   /* List of defined channels */
 static char *log_prefix_hook;             /* Name of the global prefix hook */
 static char *log_suffix_hook;             /* Name of the global suffix hook */
 
 #define SP(p) ((p)?(p):"")
 
+struct log_data {
+	int cat;
+	int pri;
+	const grad_request_t *req;
+	const char *prefix;
+	const char *text;
+	const char *errtext;
+};
+
+
+
+struct logbuf {
+	char *ptr;
+	size_t size;
+	size_t pos;
+};
+
+#define LOGBUF_INIT(b) { (b), sizeof(b), 0 }
+#define _log_s_cat2__(a,b) a ## b 
+#define LOGBUF_DECL(name,size) \
+  static char _log_s_cat2__(name,_buffer)[ size ];\
+  struct logbuf name = LOGBUF_INIT(_log_s_cat2__(name,_buffer))
+
+void
+logbuf_append(struct logbuf *buf, const char *str)
+{
+	size_t length = strlen(str);
+	size_t rest = buf->size - buf->pos;
+
+	if (rest <= 1) 
+		buf->ptr[buf->pos-1] = '>';
+	else if (rest == 2)
+		buf->ptr[buf->pos++] = '>';
+	else if (length >= rest) {
+		if (--rest > 1) {
+			if (rest >= 2)
+				rest--;
+			memcpy(buf->ptr + buf->pos, str, rest);
+			buf->pos += rest;
+		} 
+		buf->ptr[buf->pos++] = '>';
+	} else {
+		memcpy(buf->ptr + buf->pos, str, length);
+		buf->pos += length;
+	}
+}
+
+void
+logbuf_append_line(struct logbuf *buf, size_t line)
+{
+	char linestr[64];
+	snprintf(linestr, sizeof(linestr), "%lu", (unsigned long)line);
+	logbuf_append(buf, linestr);
+}
+
+void
+logbuf_vformat(struct logbuf *buf, const char *fmt, va_list ap)
+{
+	size_t rest = buf->size - buf->pos;
+	size_t length;
+	int n;
+	
+	if (rest <= 1)
+		return;
+
+	n = vsnprintf(buf->ptr + buf->pos, rest, fmt, ap);
+	length = strlen (buf->ptr + buf->pos);
+	buf->pos += length;
+	if (n == -1 || length < n)
+		buf->ptr[buf->pos-1] = '>';
+}
+
+char *
+logbuf_ptr(struct logbuf *buf)
+{
+	buf->ptr[buf->pos] = 0;
+	return buf->ptr;
+}
+
+size_t
+logbuf_printable_length(struct logbuf *buf)
+{
+	return buf->pos;
+}
+
+
 static int
 log_get_category()
 {
@@ -83,28 +170,34 @@ static char *priname[] = { /* priority names */
 };
 
 static char *
-run_log_hook(const RADIUS_REQ *req, const char *hook_name)
+run_log_hook(const grad_request_t *req, const char *hook_name)
 {
-	char *result;
-	char nasbuf[MAX_LONGNAME];
-	
+	grad_value_t val;
+	char nasbuf[GRAD_MAX_LONGNAME];
+
+	memset(&val, 0, sizeof(val));
+
+	/* FIXME: Should make sure that the hook does not modify the
+	   request, either by passing rewrite_invoke a copy of the
+	   latter (expensive), or by providing some internal rewrite
+	   mechanism */
 	if (rewrite_invoke(String,
-			   &result,
+			   &val,
 			   hook_name,
 			   req,
 			   "isi",
 			   req->code,
-			   nas_request_to_name(req, nasbuf, sizeof nasbuf),
+			   grad_nas_request_to_name(req,
+						    nasbuf, sizeof nasbuf),
 			   req->id))
 		return NULL;
-	return result;
+	return val.datum.sval.data;
 }
 
 static void
-channel_format_prefix(char **bufp, Channel *chan, const RADIUS_REQ *req)
+log_format_hook(struct logbuf *bufp, char **hook_name_ptr,
+		const grad_request_t *req)
 {
-	char **hook_name_ptr = &(chan->prefix_hook ? 
-		                   chan->prefix_hook : log_prefix_hook);
 	char *hook_res = NULL;
 
 	if (*hook_name_ptr) {
@@ -113,25 +206,10 @@ channel_format_prefix(char **bufp, Channel *chan, const RADIUS_REQ *req)
 			*hook_name_ptr = NULL;
 	}
 
-	if (hook_res)  
-		asprintf(bufp, "%s", hook_res);
-}
-
-static void
-channel_format_suffix(char **bufp, Channel *chan, const RADIUS_REQ *req)
-{
-	char **hook_name_ptr = &(chan->suffix_hook ? 
-		                   chan->suffix_hook : log_suffix_hook);
-	char *hook_res = NULL;
-	
-	if (*hook_name_ptr) {
-		hook_res = run_log_hook(req, *hook_name_ptr);
-		if (!hook_res) 
-			*hook_name_ptr = NULL;
+	if (hook_res) {
+		logbuf_append(bufp, hook_res);
+		grad_free(hook_res);
 	}
-
-	if (hook_res)  
-		asprintf(bufp, "%s", hook_res);
 }
 
 static FILE *
@@ -152,40 +230,41 @@ channel_close_file(Channel *chan, FILE *fp)
                 fclose(fp);
 }
 
-void
-log_to_channel(Channel *chan, int cat, int pri,
-	       const RADIUS_REQ *req,
-	       char *buf1, char *buf2, char *buf3)
+int
+log_to_channel(void *item, void *pdata)
 {
-        char *cat_pref = NULL;
-        char *prefix = NULL;
-	char *req_prefix_buf = NULL, *req_suffix_buf = NULL;
+	Channel *chan = item;
+	struct log_data *data = pdata;
+
+	LOGBUF_DECL(pri_prefix, 64);
+	LOGBUF_DECL(req_prefix, 256);
+	LOGBUF_DECL(req_suffix, 256);
+	
         time_t  timeval;
         char buffer[256];
         struct tm *tm, tms;
         int spri;
         FILE *fp;
-        
-        if (chan->options & LO_CAT)
-                asprintf(&cat_pref, "%s", _(catname[cat]));
-        if (chan->options & LO_PRI) {
-                if (cat_pref)
-                        asprintf(&prefix, "%s.%s",
-				 cat_pref,
-				 _(priname[pri]));
-                else
-                        asprintf(&prefix, "%s", _(priname[pri]));
-        } else if (cat_pref) {
-                prefix = cat_pref;
-                cat_pref = NULL;
-        }
 
-        if (cat_pref)
-                free(cat_pref);
+	if (!(chan->pmask[data->cat] & L_MASK(data->pri)))
+		return 0;
+	
+        if (chan->options & LO_CAT) {
+		logbuf_append(&pri_prefix, _(catname[data->cat]));
+		logbuf_append(&pri_prefix, ".");
+	}
+        if (chan->options & LO_PRI)
+		logbuf_append(&pri_prefix, _(priname[data->pri]));
 
-  	if (req) {
-		channel_format_prefix(&req_prefix_buf, chan, req);
-		channel_format_suffix(&req_suffix_buf, chan, req);
+  	if (data->req) {
+		log_format_hook(&req_prefix,
+				chan->prefix_hook ?
+ 				   &chan->prefix_hook : &log_prefix_hook,
+				data->req);
+ 		log_format_hook(&req_suffix,
+				chan->suffix_hook ?
+				   &chan->suffix_hook : &log_suffix_hook,
+				data->req);
 	}
 	
         switch (chan->mode) {
@@ -211,18 +290,18 @@ log_to_channel(Channel *chan, int cat, int pri,
                 fprintf(fp, "%s ", buffer);
                 if (chan->options & LO_PID) 
                         fprintf(fp, "[%lu]: ", (u_long) getpid());
-                if (prefix)
-                        fprintf(fp, "%s: ", prefix);
-                if (buf1)
-                        fprintf(fp, "%s", buf1);
-		if (req_prefix_buf)
-			fprintf(fp, "%s", req_prefix_buf);
-                if (buf2)
-                        fprintf(fp, "%s", buf2);
-                if (buf3)
-                        fprintf(fp, "%s", buf3);
-		if (req_suffix_buf)
-			fprintf(fp, "%s", req_suffix_buf);
+                if (logbuf_printable_length(&pri_prefix))
+                        fprintf(fp, "%s: ", logbuf_ptr(&pri_prefix));
+                if (data->prefix)
+                        fprintf(fp, "%s", data->prefix);
+		if (logbuf_printable_length(&req_prefix))
+			fprintf(fp, "%s", logbuf_ptr(&req_prefix));
+                if (data->text)
+                        fprintf(fp, "%s", data->text);
+                if (data->errtext)
+                        fprintf(fp, ": %s", data->errtext);
+		if (logbuf_printable_length(&req_suffix))
+			fprintf(fp, "%s", logbuf_ptr(&req_suffix));
                 fprintf(fp, "\n");
                 channel_close_file(chan, fp);
                 break;
@@ -231,44 +310,60 @@ log_to_channel(Channel *chan, int cat, int pri,
                 spri = chan->id.prio;
                 if (chan->options & LO_PID)
                         spri |= LOG_PID;
-                if (prefix)
-                        syslog(spri, "%s: %s%s%s%s%s",
-			       prefix,
-			       SP(buf1),
-			       SP(req_prefix_buf),
-			       SP(buf2), SP(buf3),
-			       SP(req_suffix_buf));
-                else
-                        syslog(spri, "%s%s%s%s%s",
-                               SP(buf1),
-			       SP(req_prefix_buf),
-			       SP(buf2), SP(buf3),
-			       SP(req_suffix_buf));
+                if (logbuf_printable_length(&pri_prefix)) {
+			if (data->errtext)
+				syslog(spri, "%s: %s%s%s: %s%s",
+				       logbuf_ptr(&pri_prefix),
+				       SP(data->prefix),
+				       logbuf_ptr(&req_prefix),
+				       SP(data->text),
+				       data->errtext,
+				       logbuf_ptr(&req_suffix));
+			else
+				syslog(spri, "%s: %s%s%s%s",
+				       logbuf_ptr(&pri_prefix),
+				       SP(data->prefix),
+				       logbuf_ptr(&req_prefix),
+				       SP(data->text),
+				       logbuf_ptr(&req_suffix));
+		} else {
+			if (data->errtext)
+				syslog(spri, "%s%s%s: %s%s",
+				       SP(data->prefix),
+				       logbuf_ptr(&req_prefix),
+				       SP(data->text),
+				       data->errtext,
+				       logbuf_ptr(&req_suffix));
+			else
+				syslog(spri, "%s%s%s%s",
+				       SP(data->prefix),
+				       logbuf_ptr(&req_prefix),
+				       SP(data->text),
+				       logbuf_ptr(&req_suffix));
+		}
                 break;
         }
-        
-        if (prefix)
-                free(prefix);
-	if (req_prefix_buf)
-		free(req_prefix_buf);
-	if (req_suffix_buf)
-		free(req_suffix_buf);
+	return 0;
 }
 
+/* Note: if modifying this function, make sure it does not allocate any
+   memory! */
 void
-vlog(int level,
-     const RADIUS_REQ *req,
-     const LOCUS *loc,
-     const char *func_name,
-     int en,
-     const char *fmt, va_list ap)
+radiusd_logger(int level,
+	       const grad_request_t *req,
+	       const grad_locus_t *loc,
+	       const char *func_name,
+	       int en,
+	       const char *fmt, va_list ap)
 {
         Channel *chan;
         int cat, pri;
-        char *buf1 = NULL;
-        char *buf2 = NULL;
-        char *buf3 = NULL;
-        ITERATOR *itr = iterator_create(chanlist);
+	struct log_data log_data;
+	
+	LOGBUF_DECL(buf1, 256);
+	LOGBUF_DECL(buf2, 1024);
+	
+        char *errstr = NULL;
 
         cat = L_CAT(level);
         if (cat == 0)
@@ -276,59 +371,51 @@ vlog(int level,
         pri = L_PRI(level);
         
         if (loc) {
-		if (func_name)
-			asprintf(&buf1,
-				 "%s:%d:%s: ", loc->file, loc->line,
-				 func_name);
-		else
-			asprintf(&buf1,
-				 "%s:%d: ", loc->file, loc->line);
+		logbuf_append(&buf1, loc->file);
+		logbuf_append(&buf1, ":");
+		logbuf_append_line(&buf1, loc->line);
+		if (func_name) {
+			logbuf_append(&buf1, ":");			
+			logbuf_append(&buf1, func_name);
+		}
+		logbuf_append(&buf1, ": ");			
 	}
 	
         if (en)
-                asprintf(&buf3, ": %s", strerror(en));
+                errstr = strerror(en);
 
-        vasprintf(&buf2, fmt, ap);
-        
-        for (chan = iterator_first(itr); chan; chan = iterator_next(itr)) {
-                /* Skip channels whith incorrect priority */
-                if (chan->pmask[cat] & L_MASK(pri))
-                        log_to_channel(chan, cat, pri, req, buf1, buf2, buf3);
-        }
-        iterator_destroy(&itr);
+        logbuf_vformat(&buf2, fmt, ap);
 
-        if (buf1)
-                free(buf1);
-        if (buf2)
-                free(buf2);
-        if (buf3)
-                free(buf3);     
+        log_data.cat = cat;
+	log_data.pri = pri;
+	log_data.req = req;
+	log_data.prefix = logbuf_ptr(&buf1);
+	log_data.text = logbuf_ptr(&buf2);
+	log_data.errtext = errstr;
+
+ 	grad_list_iterate(chanlist, log_to_channel, &log_data);
 }
 
 /* Interface */
 
 #ifdef USE_SQL
-/*PRINTFLIKE2*/
 void
-sqllog(int status, char *msg, ...)
+sqllog(int status, char *query)
 {
-        va_list ap;
         FILE *fp;
         char *path;
         char *filename;
 
         filename = status ? "sql-lost" : "sql.log";
-        path = mkfilename(radacct_dir, filename);
+        path = grad_mkfilename(grad_acct_dir, filename);
         if ((fp = fopen(path, "a")) == NULL) {
-                radlog(L_ERR|L_PERROR, _("could not append to file %s"), path);
-                efree(path);
+                grad_log(L_ERR|L_PERROR,  
+                         _("could not append to file %s"), path);
+                grad_free(path);
                 return;
         }
-        efree(path);
-	va_start(ap, msg);
-        vfprintf(fp, msg, ap);
-        fprintf(fp, ";\n");
-        va_end(ap);
+        grad_free(path);
+        fprintf(fp, "%s;\n", query);
         fclose(fp);
 }
 #endif
@@ -338,18 +425,18 @@ sqllog(int status, char *msg, ...)
 void
 channel_free(Channel *chan)
 {
-        efree(chan->name);
+        grad_free(chan->name);
         if (chan->mode == LM_FILE)
-                efree(chan->id.file);
-	efree(chan->prefix_hook);
-	efree(chan->suffix_hook);
-        efree(chan);
+                grad_free(chan->id.file);
+	grad_free(chan->prefix_hook);
+	grad_free(chan->suffix_hook);
+        grad_free(chan);
 }
 
 Channel *
 log_mark()
 {
-        return list_item(chanlist, 0);
+        return grad_list_item(chanlist, 0);
 }
 
 void
@@ -357,14 +444,14 @@ log_release(Channel *chan)
 {
         Channel *cp;
         int emerg, alert, crit;
-	ITERATOR *itr = iterator_create(chanlist);
+	grad_iterator_t *itr = grad_iterator_create(chanlist);
 
-	for (cp = iterator_first(itr); cp; cp = iterator_next(itr))
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr))
 		if (cp == chan)
 			break;
-        for (; cp; cp = iterator_next(itr)) {
+        for (; cp; cp = grad_iterator_next(itr)) {
                 if (!(cp->options & LO_PERSIST)) {
-			list_remove(chanlist, cp, NULL);
+			grad_list_remove(chanlist, cp, NULL);
                         channel_free(cp);
                 }
         }
@@ -374,7 +461,7 @@ log_release(Channel *chan)
         emerg = L_EMERG;
         alert = L_ALERT;
         crit  = L_CRIT;
-	for (cp = iterator_first(itr); cp; cp = iterator_next(itr)) {
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr)) {
                 int i;
                 for (i = 1; i < L_NCAT; i++) {
                         if (emerg && (cp->pmask[i] & L_MASK(emerg)))
@@ -385,7 +472,7 @@ log_release(Channel *chan)
                                 crit = 0;
                 }
         }
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
         if (emerg || alert || crit)
                 log_set_default("##emerg##", -1, emerg|alert|crit);
 }
@@ -395,17 +482,17 @@ log_change_owner(RADIUS_USER *usr)
 {
         Channel *cp;
 	int errcnt = 0;
-	ITERATOR *itr = iterator_create(chanlist);
-	for (cp = iterator_first(itr); cp; cp = iterator_next(itr)) {
+	grad_iterator_t *itr = grad_iterator_create(chanlist);
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr)) {
 		if (cp->mode == LM_FILE
 		    && chown(cp->id.file, usr->uid, usr->gid)) {
-			radlog(L_ERR,
-			       _("%s: cannot change owner to %d:%d"),
-			       cp->id.file, usr->uid, usr->gid);
+			grad_log(L_ERR,
+			         _("%s: cannot change owner to %d:%d"),
+			         cp->id.file, usr->uid, usr->gid);
 			errcnt++;
 		}
 	}
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
 	return errcnt;
 }
 
@@ -420,7 +507,7 @@ _chancmp(const void *item, const void *data)
 Channel *
 channel_lookup(char *name)
 {
-        return list_locate(chanlist, name, _chancmp);
+        return grad_list_locate(chanlist, name, _chancmp);
 }
 
 void
@@ -432,26 +519,27 @@ register_channel(Channel *chan)
 
         if (chan->mode == LM_FILE) {
                 if (strcmp(chan->id.file, "stdout")) {
-                        filename = mkfilename(radlog_dir ?
-                                              radlog_dir : RADLOG_DIR,
-                                              chan->id.file);
+                        filename = grad_mkfilename(grad_log_dir ?
+						   grad_log_dir : RADLOG_DIR,
+						   chan->id.file);
                         
                         /* check the accessibility of the file */
                         fp = fopen(filename, "a");
                         if (!fp) {
-                                radlog(L_CRIT|L_PERROR,
-                                       _("can't access log file `%s'"), filename);
-                                efree(filename);
-                                filename = estrdup("stdout");
+                                grad_log(L_CRIT|L_PERROR,
+                                         _("can't access log file `%s'"),
+                                         filename);
+                                grad_free(filename);
+                                filename = grad_estrdup("stdout");
                         } else
                                 fclose(fp);
                 } else
-                        filename = estrdup("stdout");
+                        filename = grad_estrdup("stdout");
         } else if (chan->mode == LM_SYSLOG) {
         } 
 
-        channel = emalloc(sizeof(*channel));
-        channel->name = estrdup(chan->name);
+        channel = grad_emalloc(sizeof(*channel));
+        channel->name = grad_estrdup(chan->name);
         channel->mode = chan->mode;
         if (chan->mode == LM_FILE)
                 channel->id.file = filename;
@@ -462,8 +550,8 @@ register_channel(Channel *chan)
 	channel->suffix_hook = chan->suffix_hook;
 	
 	if (!chanlist)
-		chanlist = list_create();
-	list_prepend(chanlist, channel);
+		chanlist = grad_list_create();
+	grad_list_prepend(chanlist, channel);
 }
 
 void
@@ -492,7 +580,7 @@ _regcat(void *item, void *data)
 }
 
 void
-register_category(int cat, int pri, RAD_LIST *clist)
+register_category(int cat, int pri, grad_list_t *clist)
 {
 	struct category_closure clos;
 
@@ -501,12 +589,12 @@ register_category(int cat, int pri, RAD_LIST *clist)
 
 	clos.cat = cat;
 	clos.pri = pri;
-	list_iterate(clist, _regcat, &clos);
+	grad_list_iterate(clist, _regcat, &clos);
 }
 
 /* Auxiliary calls */
 void
-log_set_to_console()
+log_set_to_console(int cat, int pri)
 {
         Channel chan;
         
@@ -516,7 +604,7 @@ log_set_to_console()
         chan.options = LO_CAT|LO_PRI|LO_PERSIST;
         register_channel(&chan);
 
-        register_category0(-1, -1, channel_lookup("stdout"));
+        register_category0(cat, pri, channel_lookup("stdout"));
 }
 
 void
@@ -541,13 +629,18 @@ format_exit_status(char *buffer, int buflen, int status)
 {
 	if (WIFEXITED(status)) {
 		snprintf(buffer, buflen,
+/* TRANSLATORS: the subject will always be printed before this msgid.
+   for example: "child 222 exited with status 0"
+*/
 			 _("exited with status %d"),
 			 WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
 		snprintf(buffer, buflen,
+/* TRANSLATORS: the subject will always be printed before this msgid. */
 			 _("terminated on signal %d"),
 			 WTERMSIG(status));
 	} else
+/* TRANSLATORS: the subject will always be printed before this msgid. */
 		snprintf(buffer, buflen, _("terminated"));
 }
 
@@ -560,11 +653,11 @@ static struct category_def {
 	int init;
 	int cat;
 	int pri;
-        RAD_LIST /* of Channel */ *clist;
+        grad_list_t /* of Channel */ *clist;
         int level;
 } cat_def;
 
-static struct keyword syslog_facility[] = {
+static grad_keyword_t syslog_facility[] = {
 	{ "user", 	LOG_USER },
 	{ "daemon", 	LOG_DAEMON },
 	{ "auth", 	LOG_AUTH },
@@ -579,7 +672,7 @@ static struct keyword syslog_facility[] = {
 	{ 0 }
 };
 
-static struct keyword syslog_priority[] = {
+static grad_keyword_t syslog_priority[] = {
 	{ "emerg", 	LOG_EMERG },
 	{ "alert", 	LOG_ALERT },
 	{ "crit", 	LOG_CRIT },
@@ -591,7 +684,7 @@ static struct keyword syslog_priority[] = {
 	{ 0 }
 };
 
-static struct keyword log_categories[] = {
+static grad_keyword_t log_categories[] = {
 	{ "main",       L_MAIN },
 	{ "auth",       L_AUTH },
 	{ "acct",       L_ACCT },
@@ -600,7 +693,7 @@ static struct keyword log_categories[] = {
 	{ 0 }
 };
 
-static struct keyword log_priorities[] = {
+static grad_keyword_t log_priorities[] = {
 	{ "emerg",      L_EMERG },
 	{ "alert",      L_ALERT },
 	{ "crit",       L_CRIT },
@@ -617,9 +710,9 @@ logging_stmt_handler(int argc, cfg_value_t *argv, void *block_data,
 		     void *handler_data)
 {
 	mark = log_mark();
-	efree(log_prefix_hook);
+	grad_free(log_prefix_hook);
 	log_prefix_hook = NULL;
-	efree(log_suffix_hook);
+	grad_free(log_suffix_hook);
 	log_suffix_hook = NULL;
 	return 0;
 }
@@ -662,9 +755,9 @@ static int
 channel_stmt_end(void *block_data, void *handler_data)
 {
 	if (channel.mode == LM_UNKNOWN) {
-		radlog(L_ERR,
-		       _("%s:%d: no channel mode for `%s'"), 
-		       cfg_filename, cfg_line_num, channel.name);
+		grad_log(L_ERR,
+		         _("%s:%d: no channel mode for `%s'"), 
+		         cfg_filename, cfg_line_num, channel.name);
 	} else 
 		register_channel(&channel);
 	return 0;
@@ -675,9 +768,9 @@ get_priority(cfg_value_t *argv)
 {
 	if (argv[0].type != CFG_CHAR || argv[1].type != CFG_STRING)
 		return 1;
-	cat_def.pri = xlat_keyword(log_priorities,
-				   argv[1].v.string,
-				   -1);
+	cat_def.pri = grad_xlat_keyword(log_priorities,
+					argv[1].v.string,
+					-1);
 	if (cat_def.pri == -1)
 		return 1;
 
@@ -715,12 +808,12 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) {
-				cat_def.pri = xlat_keyword(log_priorities,
-							   argv[1].v.string,
-							   -1);
+				cat_def.pri = grad_xlat_keyword(log_priorities,
+							      argv[1].v.string,
+								-1);
 				if (cat_def.pri == -1)
 					return 1;
 				cat_def.pri = L_UPTO(cat_def.pri);
@@ -746,8 +839,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) 
 				return 1;
 			break;
@@ -765,8 +858,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.pri = xlat_keyword(log_priorities,
-						   argv[3].v.string, -1);
+			cat_def.pri = grad_xlat_keyword(log_priorities,
+							argv[3].v.string, -1);
 			if (cat_def.pri == -1) 
 				return 1;
 			cat_def.pri = L_UPTO(cat_def.pri);
@@ -790,8 +883,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) 
 				return 1;
 			break;
@@ -823,13 +916,13 @@ category_stmt_end(void *block_data, void *handler_data)
 			break;
 		default:
 			if (cat_def.level)
-				radlog(L_WARN,
-				       "%s:%d: %s",
-				       cfg_filename, cfg_line_num,
+				grad_log(L_WARN,
+				         "%s:%d: %s",
+				         cfg_filename, cfg_line_num,
 				_("no levels applicable for this category"));
 		}
 		register_category(cat_def.cat, cat_def.pri, cat_def.clist);
-		list_destroy(&cat_def.clist, NULL, NULL);
+		grad_list_destroy(&cat_def.clist, NULL, NULL);
 	}
 	return 0;
 }
@@ -851,13 +944,13 @@ category_set_channel(int argc, cfg_value_t *argv,
 	channel = channel_lookup(argv[1].v.string);
 
 	if (!channel) {
-		radlog(L_ERR,
-		       _("%s:%d: channel `%s' not defined"),
-		       cfg_filename, cfg_line_num, argv[1].v.string);
+		grad_log(L_ERR,
+		         _("%s:%d: channel `%s' not defined"),
+		         cfg_filename, cfg_line_num, argv[1].v.string);
 	} else {
 		if (!cat_def.clist)
-			cat_def.clist = list_create();
-		list_append(cat_def.clist, channel);
+			cat_def.clist = grad_list_create();
+		grad_list_append(cat_def.clist, channel);
 	}
 	
 	return 0;
@@ -889,16 +982,16 @@ category_set_level(int argc, cfg_value_t *argv,
 {
 	int i;
 
-	clear_debug();
+	grad_clear_debug();
 	for (i = 1; i < argc; ) {
 		char *modname;
 		int level;
 		
 		if (argv[i].type != CFG_STRING) {
-			radlog(L_ERR,
-			       _("%s:%d: list item %d has wrong datatype"),
-			       cfg_filename, cfg_line_num,
-			       i);
+			grad_log(L_ERR,
+			         _("%s:%d: list item %d has wrong datatype"),
+			         cfg_filename, cfg_line_num,
+			         i);
 			return 1;
 		}
 		modname = argv[i++].v.string;
@@ -910,10 +1003,10 @@ category_set_level(int argc, cfg_value_t *argv,
 				return 1;
 			level = argv[i++].v.number;
 		}
-		if (set_module_debug_level(modname, level)) {
-			radlog(L_WARN,
-			       _("%s:%d: no such module name: %s"),
-			       cfg_filename, cfg_line_num, modname);
+		if (grad_set_module_debug_level(modname, level)) {
+			grad_log(L_WARN,
+			         _("%s:%d: no such module name: %s"),
+			         cfg_filename, cfg_line_num, modname);
 		}
 	}
 	return 0;
@@ -954,7 +1047,8 @@ channel_syslog_handler(int argc, cfg_value_t *argv, void *block_data,
 		break;
 
 	case CFG_STRING:
-		facility = xlat_keyword(syslog_facility, argv[1].v.string, -1);
+		facility = grad_xlat_keyword(syslog_facility,
+					     argv[1].v.string, -1);
 		break;
 
 	default:
@@ -973,7 +1067,8 @@ channel_syslog_handler(int argc, cfg_value_t *argv, void *block_data,
 		break;
 
 	case CFG_STRING:
-		prio = xlat_keyword(syslog_priority, argv[3].v.string, -1);
+		prio = grad_xlat_keyword(syslog_priority,
+					 argv[3].v.string, -1);
 		break;
 
 	default:
