@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -17,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_PROXY_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -44,47 +42,48 @@
 
 /* Decode a password and encode it again. */
 static void
-passwd_recode(VALUE_PAIR *pass_pair, char *new_secret, char *new_vector,
-	      RADIUS_REQ *req)
+passwd_recode(grad_avp_t *pass_pair, char *new_secret, char *new_authenticator,
+	      radiusd_request_t *req)
 {
-        char password[AUTH_STRING_LEN+1];
-        req_decrypt_password(password, req, pass_pair);
-        efree(pass_pair->avp_strvalue);
-        encrypt_password(pass_pair, password, new_vector, new_secret);
+        char password[GRAD_STRING_LENGTH+1];
+        req_decrypt_password(password, req->request, pass_pair);
+        grad_free(pass_pair->avp_strvalue);
+        grad_encrypt_password(pass_pair, password, new_authenticator, new_secret);
         /* Don't let the cleantext hang around */
-        memset(password, 0, AUTH_STRING_LEN);
+        memset(password, 0, GRAD_STRING_LENGTH);
 }
 
 /* Decode a password and encode it again. */
 static void
-tunnel_passwd_recode(VALUE_PAIR *pass_pair, char *new_secret, char *new_vector,
-		     RADIUS_REQ *req)
+tunnel_passwd_recode(grad_avp_t *pass_pair, char *new_secret, char *new_authenticator,
+		     radiusd_request_t *req)
 {
-        char password[AUTH_STRING_LEN+1];
+        char password[GRAD_STRING_LENGTH+1];
 	u_char tag;
 	
-	decrypt_tunnel_password(password, 
-				&tag, pass_pair,
-				req->vector, req->secret);
-        efree(pass_pair->avp_strvalue);
-	encrypt_tunnel_password(pass_pair,
-				tag, password, 
-				new_vector, new_secret);
-	memset(password, 0, AUTH_STRING_LEN);
+	grad_decrypt_tunnel_password(password, 
+				     &tag, pass_pair,
+				     req->request->authenticator,
+				     req->request->secret);
+        grad_free(pass_pair->avp_strvalue);
+	grad_encrypt_tunnel_password(pass_pair,
+				     tag, password, 
+				     new_authenticator, new_secret);
+	memset(password, 0, GRAD_STRING_LENGTH);
 }
 
-VALUE_PAIR *
-proxy_request_recode(RADIUS_REQ *radreq, VALUE_PAIR *plist,
-		     u_char *secret, u_char *vector)
+grad_avp_t *
+proxy_request_recode(radiusd_request_t *radreq, grad_avp_t *plist,
+		     u_char *secret, u_char *authenticator)
 {
-	VALUE_PAIR *p;
+	grad_avp_t *p;
 
 	/* Recode password pair(s) */
 	for (p = plist; p; p = p->next) {
-		if (p->prop & AP_ENCRYPT_RFC2138)
-			passwd_recode(p, secret, vector, radreq);
-		else if (p->prop & AP_ENCRYPT_RFC2868)
-			tunnel_passwd_recode(p, secret, vector, radreq);
+		if (p->prop & GRAD_AP_ENCRYPT_RFC2138)
+			passwd_recode(p, secret, authenticator, radreq);
+		else if (p->prop & GRAD_AP_ENCRYPT_RFC2868)
+			tunnel_passwd_recode(p, secret, authenticator, radreq);
 	}
 	return plist;
 }
@@ -94,23 +93,23 @@ proxy_request_recode(RADIUS_REQ *radreq, VALUE_PAIR *plist,
  */
 
 int
-proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r)
+proxy_cmp(radiusd_request_t *qr, radiusd_request_t *r)
 {
-	VALUE_PAIR *p, *proxy_state_pair = NULL;
-	RADIUS_SERVER *server;
+	grad_avp_t *p, *proxy_state_pair = NULL;
+	grad_server_t *server;
 	
 	if (!qr->realm) {
-		debug(100,("no proxy realm"));
+		GRAD_DEBUG(100,("no proxy realm"));
 		return 1;
 	}
-	server = list_item(qr->realm->queue->servers, qr->server_no);
+	server = grad_list_item(qr->realm->queue->servers, qr->server_no);
 	if (!server) {
-		debug(100,("no proxy server"));
+		GRAD_DEBUG(100,("no proxy server"));
 		return 1;
 	}
 	
         /* Find the last PROXY_STATE attribute. */
-        for (p = r->request; p; p = p->next) {
+        for (p = r->request->avlist; p; p = p->next) {
                 if (p->attribute == DA_PROXY_STATE) 
                         proxy_state_pair = p;
         }
@@ -121,7 +120,7 @@ proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r)
 
 		state = (PROXY_STATE *)proxy_state_pair->avp_strvalue;
 	
-		debug(1,
+		GRAD_DEBUG(1,
 		      ("state: ipaddr %08x, id %u, proxy_id %u, remote_ip %08x",
 		       state->client_ip,
 		       state->id,
@@ -129,19 +128,19 @@ proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r)
 		       state->remote_ip));
 		
                 if (state->ref_ip == ref_ip
-		    && state->proxy_id == r->id
-		    && state->remote_ip == r->ipaddr) {
-			debug(10, ("(old=data) id %d %d, ipaddr %#8x %#8x, proxy_id %d %d, server_addr %#8x %#8x", 
-				   qr->id, state->id,
-				   qr->ipaddr, state->client_ip,
+		    && state->proxy_id == r->request->id
+		    && state->remote_ip == r->request->ipaddr) {
+			GRAD_DEBUG(10, ("(old=data) id %d %d, ipaddr %#8x %#8x, proxy_id %d %d, server_addr %#8x %#8x", 
+				   qr->request->id, state->id,
+				   qr->request->ipaddr, state->client_ip,
 				   qr->server_id, state->proxy_id,
 				   server->addr, state->remote_ip));
         
-			if (state->client_ip == qr->ipaddr
-			    && state->id  == qr->id
+			if (state->client_ip == qr->request->ipaddr
+			    && state->id  == qr->request->id
 			    && state->proxy_id == qr->server_id
 			    && state->remote_ip == server->addr) {
-				debug(1,("EQUAL!!!"));
+				GRAD_DEBUG(1,("EQUAL!!!"));
 				return 0;
 			}
 		} 
@@ -155,7 +154,7 @@ proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r)
 
 /* Send the request */
 static int
-proxy_send_pdu(int fd, RADIUS_SERVER *server, RADIUS_REQ *radreq,
+proxy_send_pdu(int fd, grad_server_t *server, radiusd_request_t *radreq,
 	       void *pdu, size_t size)
 {
 	struct sockaddr_in sin;
@@ -164,71 +163,76 @@ proxy_send_pdu(int fd, RADIUS_SERVER *server, RADIUS_REQ *radreq,
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = htonl(server->addr);
 	
-	sin.sin_port = htons((radreq->code == RT_ACCESS_REQUEST) ?
-			     server->port[PORT_AUTH] : server->port[PORT_ACCT]);
+	sin.sin_port = htons((radreq->request->code == RT_ACCESS_REQUEST) ?
+			     server->port[GRAD_PORT_AUTH] 
+			     : server->port[GRAD_PORT_ACCT]);
 
-	debug(1, ("Proxying id %d to %lx",
-		  radreq->id, (u_long)server->addr));
+	GRAD_DEBUG(1, ("Proxying id %d to %lx",
+		  radreq->request->id, (u_long)server->addr));
 
 	return sendto(fd, pdu, size, 0, (struct sockaddr *)&sin, sizeof(sin));
 }
 
 int
-proxy_send_request(int fd, RADIUS_REQ *radreq)
+proxy_send_request(int fd, radiusd_request_t *radreq)
 {
-	VALUE_PAIR *plist, *p;
+	grad_avp_t *plist, *p;
 	void *pdu;
 	size_t size;
-	RADIUS_SERVER *server;
+	grad_server_t *server;
 	int rc;
 	PROXY_STATE *proxy_state;
 	
-	if (radreq->attempt_no > radreq->realm->queue->retries) {
+	if (radreq->attempt_no >= radreq->realm->queue->retries) {
 		radreq->server_no++;
 		radreq->attempt_no = 0;
 	}
-	server = list_item(radreq->realm->queue->servers, radreq->server_no);
+	server = grad_list_item(radreq->realm->queue->servers,
+				radreq->server_no);
 
 	if (!server) {
-		radlog_req(L_NOTICE, radreq,
-		           _("couldn't send request to realm %s"),
-		           radreq->realm->realm);
+		grad_log_req(L_NOTICE, radreq->request,
+		             _("couldn't send request to realm %s"),
+		             radreq->realm->realm);
 		return 0;
 	}
 	if (radreq->attempt_no == 0)
-		radreq->server_id = rad_clt_message_id(server);
+		radreq->server_id = grad_client_message_id(server);
 	radreq->attempt_no++;
 
-	rad_clt_random_vector(radreq->remote_auth);
-	plist = proxy_request_recode(radreq, avl_dup(radreq->request),
-				     server->secret, radreq->remote_auth);
+	grad_client_random_authenticator(radreq->remote_auth);
+	plist = proxy_request_recode(radreq,
+				     grad_avl_dup(radreq->request->avlist),
+				     server->secret,
+				     radreq->remote_auth);
 
 	/* Add a proxy-pair to the end of the request. */
-	p = avp_alloc();
+	p = grad_avp_alloc();
 	p->name = "Proxy-State";
 	p->attribute = DA_PROXY_STATE;
-	p->type = TYPE_STRING;
+	p->type = GRAD_TYPE_STRING;
 	p->avp_strlength = sizeof(PROXY_STATE);
-	p->avp_strvalue = emalloc(p->avp_strlength);
+	p->avp_strvalue = grad_emalloc(p->avp_strlength);
 	
 	proxy_state = (PROXY_STATE *)p->avp_strvalue;
 	
 	proxy_state->ref_ip = ref_ip;
-	proxy_state->client_ip = radreq->ipaddr;
-	proxy_state->id = radreq->id;
+	proxy_state->client_ip = radreq->request->ipaddr;
+	proxy_state->id = radreq->request->id;
 	proxy_state->proxy_id = radreq->server_id;
 	proxy_state->remote_ip = server->addr;
 	
-	avl_add_pair(&plist, p);
+	grad_avl_add_pair(&plist, p);
 	
 	/* Create the pdu */
-	size = rad_create_pdu(&pdu, radreq->code,
-			      radreq->server_id,
-			      radreq->remote_auth,
-			      server->secret,
-			      plist,
-			      NULL);
-	avl_free(plist);
+	size = grad_create_pdu(&pdu,
+			       radreq->request->code,
+			       radreq->server_id,
+			       radreq->remote_auth,
+			       server->secret,
+			       plist,
+			       NULL);
+	grad_avl_free(plist);
 
 	if (!radiusd_master()) {
 		RADIUS_UPDATE *upd;
@@ -236,39 +240,39 @@ proxy_send_request(int fd, RADIUS_REQ *radreq)
 		
                 /* Prepare update data. */
 		size = sizeof(*upd) + strlen(radreq->realm->realm) + 1;
-		upd = emalloc(size);
-		upd->id = radreq->id;
+		upd = grad_emalloc(size);
+		upd->id = radreq->request->id;
 		upd->proxy_id = radreq->server_id;
 		upd->server_no = radreq->server_no;
 		strcpy(upd->realmname, radreq->realm->realm);
 
-		debug(100,
+		GRAD_DEBUG(100,
 		      ("Update id=%d, proxy_id=%d, realm=%s, server_no=%d",
 		       upd->id,upd->proxy_id,upd->realmname,
 		       upd->server_no));
 
 		rpp_update(upd, size);
-		efree(upd);
+		grad_free(upd);
 	}
 	
 	rc = proxy_send_pdu(fd, server, radreq, pdu, size);
-	efree(pdu);
+	grad_free(pdu);
 	return rc;
 }
 
 /* ************************************************************************* */
 /* Interface functions */
 
-static REALM *
-proxy_lookup_realm(RADIUS_REQ *req, char *name)
+static grad_realm_t *
+proxy_lookup_realm(radiusd_request_t *req, char *name)
 {
-	REALM *realm = realm_lookup_name(name);
+	grad_realm_t *realm = grad_realm_lookup_name(name);
 	static char *var[] = { "auth", "acct" };
 	int t;
 	
 	if (realm) {
 		int rc;
-		switch (req->code) {
+		switch (req->request->code) {
 		case RT_ACCESS_REQUEST:
 			t = R_AUTH;
 			break;
@@ -279,16 +283,16 @@ proxy_lookup_realm(RADIUS_REQ *req, char *name)
 			
 		default:
 			/* Should not happen.. */
-			insist_fail("unexpected request code");
+			grad_insist_fail("unexpected request code");
 		}
 
-		rc = envar_lookup_int(realm->args, var[t], -1);
+		rc = grad_envar_lookup_int(realm->args, var[t], -1);
 		if (rc == -1) {
 			/* Neither {var} nor no{var} is specified. Check
 			   the orthogonal variable. If it is not set, proxying
 			   is enabled for both authentication and
 			   accounting. */
-			rc = !envar_lookup_int(realm->args, var[!t], 0);
+			rc = !grad_envar_lookup_int(realm->args, var[!t], 0);
 		}
 		if (!rc)
 			realm = NULL;
@@ -305,31 +309,31 @@ proxy_lookup_realm(RADIUS_REQ *req, char *name)
 int
 proxy_send(REQUEST *req)
 {
-	RADIUS_REQ *radreq = req->data;
+	radiusd_request_t *radreq = req->data;
         char *username;
-        VALUE_PAIR *namepair;
-        VALUE_PAIR *vp;
+        grad_avp_t *namepair;
+        grad_avp_t *vp;
         char *realmname;
-        REALM *realm;
+        grad_realm_t *realm;
 
         /* Look up name. */
-        namepair = avl_find(radreq->request, DA_USER_NAME);
-        if (avp_null_string_p(namepair))
+        namepair = grad_avl_find(radreq->request->avlist, DA_USER_NAME);
+        if (grad_avp_null_string_p(namepair))
                 return 0;
 
         username = namepair->avp_strvalue;
 
         /* Find the realm from the _end_ so that we can cascade realms:
-           user@realm1@realm2. Two special realms are handled separately:
-           
-               LOCAL    -- process request locally.
-               NOREALM  -- handle an empty realm name.
+           user@realm1@realm2. A special realms NOREALM matches usual
+	   user name (without explicit realm). A realm named DEFAULT is
+	   returned by grad_realm_lookup_name() if no other realm name
+	   matches.
 
-           A realm with special name DEFAULT is returned by realm_lookup_name()
-           if no other realm name matches. */
+	   In addition, a keyword LOCAL in server list fields indicates
+	   that the realm should be handled locally */
 
         if ((realmname = strrchr(username, '@')) == NULL) {
-                if ((realm = realm_lookup_name("NOREALM")) == NULL) {
+                if ((realm = grad_realm_lookup_name("NOREALM")) == NULL) {
                         return 0;
                 }
         } else if ((realm = proxy_lookup_realm(radreq, realmname + 1))
@@ -338,7 +342,7 @@ proxy_send(REQUEST *req)
                 return 0;
         }
 
-	if (realmname && realm_strip_p(realm)) {
+	if (realmname && grad_realm_strip_p(realm)) {
 		*realmname = 0;
 		namepair->avp_strlength = strlen(namepair->avp_strvalue);
 	}
@@ -351,8 +355,12 @@ proxy_send(REQUEST *req)
 	radreq->attempt_no = 0;
 	/* Actual user name may be altered, but the remote user name
 	   should remain the same */
-	radreq->remote_user = estrdup(username); 
+	radreq->remote_user = grad_estrdup(username); 
 
+	/* Add a Realm-Name pair */
+	grad_avl_add_pair(&radreq->request->avlist,
+			  grad_avp_create_string(DA_REALM_NAME, realm->realm));
+	
         /* If there is a DA_CHAP_PASSWORD attribute, there is
 	   also a DA_CHAP_CHALLENGE. If you change the code of
 	   radius_auth_req_decode(), you will have
@@ -365,12 +373,12 @@ proxy_send(REQUEST *req)
 
 /* FIXME! server timeout is not used */
 void
-proxy_retry(RADIUS_REQ *req, int fd)
+proxy_retry(radiusd_request_t *req, int fd)
 {
-	VALUE_PAIR *namepair;
+	grad_avp_t *namepair;
 	char *saved_username;
 	
-        namepair = avl_find(req->request, DA_USER_NAME);
+        namepair = grad_avl_find(req->request->avlist, DA_USER_NAME);
         if (namepair == NULL)
                 return;
 
@@ -386,26 +394,26 @@ proxy_retry(RADIUS_REQ *req, int fd)
 }
 
 static int
-select_propagated(void *null ARG_UNUSED, VALUE_PAIR *pair)
+select_propagated(void *null ARG_UNUSED, grad_avp_t *pair)
 {
-        return pair->prop & AP_PROPAGATE;
+        return pair->prop & GRAD_AP_PROPAGATE;
 }
 
 /* Called when a response from a remote radius server has been received.
    The function finds the original request and replaces all fields in
-   radreq, except `request', with the original data.
+   radreq, except `request->avlist', with the original data.
    Return:   0 proxy found
             -1 error don't reply */
 int
-proxy_receive(RADIUS_REQ *radreq, RADIUS_REQ *oldreq, int fd)
+proxy_receive(radiusd_request_t *radreq, radiusd_request_t *oldreq, int fd)
 {
-        VALUE_PAIR *vp, *proxy_state_pair, *prev, *x;
-        VALUE_PAIR *allowed_pairs;
+        grad_avp_t *vp, *proxy_state_pair, *prev, *x;
+        grad_avp_t *allowed_pairs;
 	
         /* Remove the last proxy pair from the list. */
         proxy_state_pair = x = prev = NULL;
 
-        for (vp = radreq->request; vp; vp = vp->next) {
+        for (vp = radreq->request->avlist; vp; vp = vp->next) {
                 if (vp->attribute == DA_PROXY_STATE) {
                         prev = x;
                         proxy_state_pair = vp;
@@ -417,35 +425,38 @@ proxy_receive(RADIUS_REQ *radreq, RADIUS_REQ *oldreq, int fd)
                 if (prev)
                         prev->next = proxy_state_pair->next;
                 else
-                        radreq->request = proxy_state_pair->next;
-                avp_free(proxy_state_pair);
+                        radreq->request->avlist = proxy_state_pair->next;
+                grad_avp_free(proxy_state_pair);
         }
 
         /* Only allow some attributes to be propagated from
            the remote server back to the NAS, for security. */
         allowed_pairs = NULL;
-        avl_move_pairs(&allowed_pairs, &radreq->request,
-		       select_propagated, NULL);
-        avl_free(radreq->request);
+        grad_avl_move_pairs(&allowed_pairs, &radreq->request->avlist,
+			    select_propagated, NULL);
+        grad_avl_free(radreq->request->avlist);
 
-        /* Rebuild the RADIUS_REQ struct, so that the normal functions
+        /* Rebuild the radiusd_request_t struct, so that the normal functions
            can process it. Take care not to modify oldreq! */
 
-	memcpy(radreq->vector, oldreq->remote_auth, sizeof radreq->vector);
+	memcpy(radreq->request->authenticator,
+	       oldreq->remote_auth, sizeof radreq->request->authenticator);
  	radreq->server_reply = proxy_request_recode(radreq, allowed_pairs,
-						    oldreq->secret,
-						    oldreq->vector);
+						    oldreq->request->secret,
+						    oldreq->request->authenticator);
         radreq->validated    = 1;
-        radreq->server_code  = radreq->code;
-        radreq->code         = oldreq->code;
+        radreq->server_code  = radreq->request->code;
+        radreq->request->code = oldreq->request->code;
 
-        radreq->ipaddr       = oldreq->ipaddr;
-        radreq->udp_port     = oldreq->udp_port;
-        radreq->id           = oldreq->id;
+        radreq->request->ipaddr       = oldreq->request->ipaddr;
+        radreq->request->udp_port     = oldreq->request->udp_port;
+        radreq->request->id           = oldreq->request->id;
 
-	memcpy(radreq->vector, oldreq->vector, sizeof radreq->vector);
-        radreq->secret       = oldreq->secret;
-        radreq->request      = avl_dup(oldreq->request);
+	memcpy(radreq->request->authenticator,
+	       oldreq->request->authenticator,
+	       sizeof radreq->request->authenticator);
+        radreq->request->secret       = oldreq->request->secret;
+        radreq->request->avlist       = grad_avl_dup(oldreq->request->avlist);
 
         /* Proxy support fields */
         radreq->realm         = oldreq->realm;
