@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,42 +45,52 @@ static char doc[] = N_("GNU radius daemon");
 #define SHOW_DEFAULTS_OPTION 256
 
 static struct argp_option options[] = {
+#define GRP 100	
         {NULL, 0, NULL, 0,
-         N_("radiusd specific switches:"), 0},
+         N_("radiusd specific switches:"), GRP},
+        {"foreground", 'f', NULL, 0,
+         N_("Stay in foreground"), GRP+1},
+        {"mode", 'm', "{t|c|b}", 0,
+         N_("Select operation mode: test, checkconf, builddbm."), GRP+1},
+        {"single-process", 's', NULL, 0,
+         N_("Run in single process mode"), GRP+1},
+        {"pid-file-dir", 'P', N_("DIR"), 0,
+         N_("Store pidfile in DIR"), GRP+1},
+	{"show-defaults", SHOW_DEFAULTS_OPTION, NULL, 0,
+	 N_("Show compilation defaults"), GRP+1},
+	{"quiet", 'q', NULL, 0,
+	 N_("Quiet mode (valid only with --mode)"), GRP+1},
+#undef GRP
+#define GRP 200
+	{NULL, 0, NULL, 0,
+         N_("Daemon configuration options. Please use raddb/config instead."),
+	 GRP},
+	
         {"log-auth-detail", 'A', 0, 0,
-         N_("Do detailed authentication logging"), 0},
+         N_("Do detailed authentication logging"), GRP+1},
         {"acct-directory",  'a', N_("DIR"), 0,
-         N_("Set accounting directory"), 0},
+         N_("Set accounting directory"), GRP+1},
 #ifdef USE_DBM
         {"dbm", 'b', NULL, 0,
-         N_("Enable DBM support"), 0},
+         N_("Enable DBM support"), GRP+1},
 #endif
-        {"foreground", 'f', NULL, 0,
-         N_("Stay in foreground"), 0},
         {"logging-directory", 'l', N_("DIR"), 0, 
-         N_("Set logging directory name"), 0},
-        {"mode", 'm', "{t|c|b}", 0,
-         N_("Select operation mode: test, checkconf, builddbm.")},
+         N_("Set logging directory name"), GRP+1},
         {"do-not-resolve", 'n', NULL, 0,
-         N_("Do not resolve IP addresses"), 0},
+         N_("Do not resolve IP addresses"), GRP+1},
         {"ip-address", 'i', N_("IPADDR"), 0,
-         N_("Listen on IPADDR"), 0},
+         N_("Listen on IPADDR"), GRP+1},
         {"port", 'p', "NUMBER", 0,
-         N_("Set authentication port number"), 0},
-        {"pid-file-dir", 'P', N_("DIR"), 0,
-         N_("Store pidfile in DIR"), 0},
+         N_("Set authentication port number"), GRP+1},
         {"log-stripped-names", 'S', NULL, 0,
-         N_("Strip prefixes/suffixes off user names before logging")},
-        {"single-process", 's', NULL, 0,
-         N_("Run in single process mode"), 0},
+         N_("Strip prefixes/suffixes off user names before logging"), GRP+1},
         {"debug", 'x', N_("DEBUGSPEC"), 0,
-         N_("Set debugging level"), 0},
+         N_("Set debugging level"), GRP+1},
         {"log-auth", 'y', NULL, 0,
-         N_("Log authentications"), 0},
+         N_("Log authentications"), GRP+1},
         {"log-auth-pass", 'z', NULL, 0,
-         N_("Log users' passwords"), 0},
-	{"show-defaults", SHOW_DEFAULTS_OPTION, NULL, 0,
-	 N_("Show compilation defaults"), 0},
+         N_("Log users' passwords"), GRP+1},
+#undef GRP
         {NULL, 0, NULL, 0, NULL, 0}
 };
 
@@ -89,6 +99,8 @@ static struct argp_option options[] = {
 
 int        debug_flag;     /* can be raised from debugger only */
 int        log_mode;       /* logging mode */
+int        console_logging_priority = -1; /* Default priority for console
+					     logging */
 char       *auth_log_hook; /* Authentication logging hook function */
 
 static int foreground; /* Stay in the foreground */
@@ -178,7 +190,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'a':
-                radacct_dir = grad_estrdup(optarg);
+                grad_acct_dir = grad_estrdup(arg);
                 break;
 		
 #ifdef USE_DBM
@@ -192,7 +204,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'l':
-                radlog_dir = grad_estrdup(optarg);
+                grad_log_dir = grad_estrdup(arg);
                 break;
 		
         case 'm':
@@ -222,24 +234,28 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'n':
-                resolve_hostnames = 0;
+                grad_resolve_hostnames = 0;
                 break;
 		
         case 'i':
-                if ((myip = grad_ip_gethostaddr(optarg)) == 0)
+                if ((myip = grad_ip_gethostaddr(arg)) == 0)
                         fprintf(stderr,
                                 _("invalid IP address: %s"),
-                                optarg);
+                                arg);
                 break;
 		
         case 'P':
-                radpid_dir = optarg;
+                grad_pid_dir = arg;
                 break;
 		
         case 'p':
-                auth_port = atoi(optarg);
+                auth_port = atoi(arg);
 		acct_port = auth_port+1;
                 break;
+
+	case 'q':
+		console_logging_priority = L_ERR;
+		break;
 		
         case 'S':
                 strip_names++;  
@@ -250,8 +266,8 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'x':
-                x_debug_spec = optarg;
-                grad_set_debug_levels(optarg);
+                x_debug_spec = arg;
+                grad_set_debug_levels(arg);
                 break;
 		
         case 'y':
@@ -277,7 +293,7 @@ static struct argp argp = {
         parse_opt,
         NULL,
         doc,
-        rad_common_argp_child,
+        grad_common_argp_child,
         NULL, NULL
 };
 
@@ -504,8 +520,8 @@ radiusd_daemon()
 
         /* This is needed for messages generated by guile
            functions.
-	   FIXME: The compiled-in value of radlog_dir is used */
-        p = grad_mkfilename(radlog_dir, "radius.stderr");
+	   FIXME: The compiled-in value of grad_log_dir is used */
+        p = grad_mkfilename(grad_log_dir, "radius.stderr");
         i = open(p, O_CREAT|O_WRONLY, 0644);
         if (i != -1) {
                 if (i != 2) 
@@ -571,7 +587,7 @@ radiusd_main()
 	if (radiusd_user.username) {
 		char *p;
 		log_change_owner(&radiusd_user);
-		p = grad_mkfilename(radlog_dir, "radius.stderr");
+		p = grad_mkfilename(grad_log_dir, "radius.stderr");
 		chown(p, radiusd_user.uid, radiusd_user.gid);
 		grad_free(p);
 		radius_switch_to_user(&radiusd_user);
@@ -597,7 +613,7 @@ main(int argc, char **argv)
            It means developer is taking control in his hands, so
            we won't modify any variables that could prevent him
            from doing so. */
-        if (debug_flag == 0) {
+	if (debug_flag == 0) {
                 foreground = 0;
                 spawn_flag = 1;
         }
@@ -618,7 +634,7 @@ main(int argc, char **argv)
 
         log_set_default("default.log", -1, -1);
         if (radius_mode != MODE_DAEMON)
-                log_set_to_console();
+                log_set_to_console(-1, console_logging_priority);
 
 	radiusd_setup();
 	radiusd_start();
@@ -935,7 +951,7 @@ radiusd_reconfigure()
 	
 	grad_log(L_INFO, _("Loading configuration files."));
 	/* Read main configuration file */
-        filename = grad_mkfilename(radius_dir, RADIUS_CONFIG);
+        filename = grad_mkfilename(grad_config_dir, RADIUS_CONFIG);
         cfg_read(filename, config_syntax, NULL);
 	grad_free(filename);
 
@@ -1012,7 +1028,7 @@ void
 radiusd_pidfile_write(char *name)
 {
         pid_t pid = getpid();
-        char *p = grad_mkfilename(radpid_dir, name);
+        char *p = grad_mkfilename(grad_pid_dir, name);
 	FILE *fp = fopen(p, "w"); 
 	if (fp) {
                 fprintf(fp, "%lu\n", (u_long) pid);
@@ -1025,7 +1041,7 @@ pid_t
 radiusd_pidfile_read(char *name)
 {
 	unsigned long val;
-	char *p = grad_mkfilename(radpid_dir, name);
+	char *p = grad_mkfilename(grad_pid_dir, name);
 	FILE *fp = fopen(p, "r");
 	if (!fp)
 		return -1;
@@ -1039,7 +1055,7 @@ radiusd_pidfile_read(char *name)
 void
 radiusd_pidfile_remove(char *name)
 {
-	char *p = grad_mkfilename(radpid_dir, name);
+	char *p = grad_mkfilename(grad_pid_dir, name);
 	unlink(p);
 	grad_free(p);
 }
@@ -1319,11 +1335,11 @@ struct cfg_stmt option_stmt[] = {
 	  &exec_user, NULL, NULL },
 	{ "radiusd-user", CS_STMT, NULL, rad_cfg_user,
 	  &radiusd_user, NULL, NULL },
-	{ "log-dir", CS_STMT, NULL, cfg_get_string, &radlog_dir,
+	{ "log-dir", CS_STMT, NULL, cfg_get_string, &grad_log_dir,
 	  NULL, NULL },
-	{ "acct-dir", CS_STMT, NULL, cfg_get_string, &radacct_dir,
+	{ "acct-dir", CS_STMT, NULL, cfg_get_string, &grad_acct_dir,
 	  NULL, NULL },
-	{ "resolve", CS_STMT, NULL, cfg_get_boolean, &resolve_hostnames,
+	{ "resolve", CS_STMT, NULL, cfg_get_boolean, &grad_resolve_hostnames,
 	  NULL, NULL },
 	{ "username-chars", CS_STMT, NULL, cfg_get_string,
 	  &username_valid_chars, NULL, NULL },
