@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius
-   Copyright (C) 2000,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -49,7 +49,7 @@ obstack_grow_escaped(struct obstack *obp, char *str, int len)
 				snprintf(buf, sizeof(buf), "%03o",
 					 *(u_char*)str);
 				obstack_1grow(obp, '\\');
-				obstack_grow(obp, str, 3);
+				obstack_grow(obp, buf, 3);
 			}
                 }
         }
@@ -477,19 +477,19 @@ util_xlate(struct obstack *sp, char *fmt, grad_request_t *radreq)
 	char *str;
 	
 	if (fmt[0] == '=') {
-		Datatype type;
-		Datum datum;
-
+		grad_value_t val;
+		
 		/*FIXME: Should be compiled!*/
-		if (rewrite_interpret(fmt+1, radreq, &type, &datum)) 
+		if (rewrite_interpret(fmt+1, radreq, &val)) 
 			return NULL;
-		if (type != String) {
+		if (val.type != String) {
 			grad_log(L_ERR, "%s: %s",
 			         fmt+1, _("wrong return type"));
+			/* Nothing to free in val */
 			return NULL;
 		}
-		obstack_grow(sp, datum.sval, strlen(datum.sval)+1);
-		grad_free(datum.sval);
+		obstack_grow(sp, val.datum.sval.data, val.datum.sval.size + 1);
+		grad_value_free(&val);
 		str = obstack_finish(sp);
 	} else {
 		str = radius_xlate(sp, fmt, radreq, NULL);
@@ -498,61 +498,114 @@ util_xlate(struct obstack *sp, char *fmt, grad_request_t *radreq)
 }
 
 static void
-pair_set_value(grad_avp_t *p, Datatype type, Datum *datum)
+pair_set_value(grad_avp_t *p, grad_value_t *val)
 {
-	grad_free(p->avp_strvalue);
-	switch (type) {
+	char buf[64];
+	char *endp;
+	
+	switch (val->type) {
 	case Integer:
-		p->avp_lvalue = datum->ival;
+		switch (p->type) {
+		case GRAD_TYPE_STRING:
+			snprintf(buf, sizeof buf, "%lu", val->datum.ival);
+			grad_string_replace(&p->avp_strvalue, buf);
+			p->avp_strlength = strlen(p->avp_strvalue);
+			break;
+			
+		case GRAD_TYPE_INTEGER:
+		case GRAD_TYPE_IPADDR:
+		case GRAD_TYPE_DATE:
+			p->avp_lvalue = val->datum.ival;
+		}
 		break;
 		
 	case String:
-		p->avp_strvalue = datum->sval;
-		p->avp_strlength = strlen(p->avp_strvalue);
+		switch (p->type) {
+		case GRAD_TYPE_STRING:
+			grad_string_replace(&p->avp_strvalue,
+					    val->datum.sval.data);
+			p->avp_strlength = strlen(p->avp_strvalue);
+			break;
+
+		case GRAD_TYPE_INTEGER:
+		case GRAD_TYPE_IPADDR:
+		case GRAD_TYPE_DATE:
+			p->avp_lvalue = strtoul(val->datum.sval.data, &endp, 0);
+			if (*endp)
+				grad_log(L_ERR,
+					 _("cannot convert \"%s\" to integer"),
+					 val->datum.sval);
+			break;
+		}
 		break;
 		
 	default:
-		grad_insist_fail("bad Datatype");
+		grad_insist_fail("bad datatype");
 	}
 	p->eval_type = grad_eval_const;
 }
 
+/* Evaluate an A/P pair P in the context of RADIUS request REQ. If ALLOW_XLATE
+   is true, constant pairs will be evaluated using traditional method, a.k.a.
+   radius_xlate. In this case REPLY supplies optional reply pairs.
+
+   FIXME: ALLOW_XLATE is actually a kludge (see function radius_eval_avl
+   below), and REPLY could be substituted by req->reply. */ 
+int
+radius_eval_avp(radiusd_request_t *req, grad_avp_t *p, grad_avp_t *reply,
+		int allow_xlate)
+{
+	grad_value_t val;
+	
+	switch (p->eval_type) {
+	case grad_eval_const:
+		if (allow_xlate && strchr(p->avp_strvalue, '%')) {
+			struct obstack s;
+			char *ptr;
+			obstack_init (&s);
+			ptr = radius_xlate(&s, p->avp_strvalue,
+					   req->request, reply);
+			if (strcmp(ptr, p->avp_strvalue)) {
+				grad_string_replace(&p->avp_strvalue, ptr);
+				p->avp_strlength = strlen (p->avp_strvalue);
+			}
+			obstack_free (&s, NULL);
+		}
+		break;
+			
+	case grad_eval_interpret:
+		if (rewrite_interpret(p->avp_strvalue, req->request, &val)) 
+			return 1;
+		pair_set_value(p, &val);
+		grad_value_free(&val);
+		break;
+
+	case grad_eval_compiled:
+		if (rewrite_eval(p->avp_strvalue, req->request, &val))
+			return 1;
+		pair_set_value(p, &val);
+		grad_value_free(&val);
+		break;
+
+	default:
+		grad_insist_fail("bad eval_type");
+		return 1;
+	}
+	p->eval_type = grad_eval_const;
+	return 0;
+}
+
+/* Evaluate A/V pair list P in the context of RADIUS request REQ.
+   For backward compatibility, no traditional attribute translation is
+   performed, this is actually the reason for existence of the last
+   argument to radius_eval_avp(). This will change in future versions. */
 int
 radius_eval_avl(radiusd_request_t *req, grad_avp_t *p)
 {
-	int errcnt = 0;
-	for (; p; p = p->next) {
-		Datatype type;
-		Datum datum;
-		
-		switch (p->eval_type) {
-		case grad_eval_const:
-			break;
-			
-		case grad_eval_interpret:
-			if (rewrite_interpret(p->avp_strvalue,
-					      req->request, &type, &datum)) {
-                                errcnt++;
-                                continue;
-                        }
-			pair_set_value(p, type, &datum);
-			break;
-
-		case grad_eval_compiled:
- 			if (rewrite_eval(p->avp_strvalue,
-					 req->request, &type, &datum)) {
-                                errcnt++;
-                                continue;
-                        }
-			pair_set_value(p, type, &datum);
-			break;
-
-		default:
-			grad_insist_fail("bad eval_type");
-                }
-        }
-
-	return errcnt++;
+	int status = 0;
+	for (; p; p = p->next)
+		status |= radius_eval_avp(req, p, NULL, 0);
+	return status;
 }
 
 radiusd_request_t *
