@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -105,7 +105,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
 {
         switch (key) {
         case 'a':
-                assign(optarg);
+                assign(arg);
                 break;
 		
         case 'q':
@@ -113,15 +113,15 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'r':
-                retry = strtol(optarg, NULL, 0);
+                retry = strtol(arg, NULL, 0);
                 break;
 		
         case 's':
-                server = optarg;
+                server = arg;
                 break;
 		
         case 'f':
-                filename = optarg;
+                filename = arg;
 		*(int *)state->input = state->next;
 		state->next = state->argc;
                 break;
@@ -135,11 +135,11 @@ parse_opt(int key, char *arg, struct argp_state *state)
 		break;
 		
         case 't':
-                timeout = strtol(optarg, NULL, 0);
+                timeout = strtol(arg, NULL, 0);
                 break;
 		
         case 'x':
-                grad_set_debug_levels(optarg);
+                grad_set_debug_levels(arg);
                 break;
 		
         case 'v':
@@ -159,7 +159,7 @@ static struct argp argp = {
         parse_opt,
         NULL,
         doc,
-        rad_common_argp_child,
+        grad_common_argp_child,
         NULL,
 	NULL
 };
@@ -241,7 +241,7 @@ main(int argc, char **argv)
                 int i, argc;
                 char **argv;
 
-                if (argcv_get(server, ":", NULL, &argc, &argv)) {
+                if (grad_argcv_get(server, ":", NULL, &argc, &argv)) {
                         grad_log(L_ERR, _("can't parse server definition"));
                         exit(1);
                 }
@@ -306,7 +306,7 @@ main(int argc, char **argv)
                         serv.port[1] = DEF_ACCT_PORT;
 		grad_client_append_server(srv_queue,
 					  grad_client_alloc_server(&serv));
-                argcv_free(argc, argv);
+                grad_argcv_free(argc, argv);
         }
 
         if (grad_list_count(srv_queue->servers) == 0) {
